import { Component, OnInit, OnDestroy, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Subscription } from 'rxjs';
import { JhiEventManager } from 'ng-jhipster';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { IBcsNgListOption, ListOptionTypeValuesAndLabels } from '../../../bcs_ng/model/bcs-ng-list-option.model';

import { BcsNgListOptionService } from './bcs-ng-list-option.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { AccountService } from 'src/app/core/auth/account.service';
import { MatTableDataSource } from '@angular/material/table';
import { MatPaginator } from '@angular/material/paginator';
import { DialogEraseComponent } from 'src/app/shared/dialog/erase/dialog-erase.component';
import { DialogRebornComponent } from 'src/app/shared/dialog/reborn/dialog-reborn.component';
import { DialogDeleteComponent } from 'src/app/shared/dialog/delete/dialog-delete.component';
import { MatSort } from '@angular/material/sort';
import { BcsNgListOptionUpdateComponent } from './bcs-ng-list-option-update.component';
import { IBcsNgListOptionExcel } from '../../shared/dto/model/bcs-ng-list-option.excel.model';
import { ExcelService } from 'src/app/shared/util/excel.service';

@Component({
  selector: 'jhi-bcs-ng-list-option',
  templateUrl: './bcs-ng-list-option.component.html',
})
export class BcsNgListOptionComponent extends BaseComponent implements OnInit, OnDestroy {
  bcsNgListOptions?: IBcsNgListOption[];
  listOptionTypeLabelsAndValues = ListOptionTypeValuesAndLabels;

  eventSubscriber?: Subscription;
  load = false;
  dataSource = new MatTableDataSource<IBcsNgListOption>();
  @ViewChild(MatPaginator, { static: true }) paginator!: MatPaginator;
  @ViewChild(MatSort) matSorter!: MatSort;
  public pageSize = 25;
  public currentPage = 0;

  constructor(
    protected bcsNgListOptionService: BcsNgListOptionService,
    protected activatedRoute: ActivatedRoute,
    protected router: Router,
    protected eventManager: JhiEventManager,
    protected modalService: NgbModal,
    protected excelService: ExcelService,
    protected accountService: AccountService,
  ) {
    super(accountService);
  }

  loadPage(): void {
    this.load = true;
    this.bcsNgListOptionService
      .query()
      .subscribe(
        res => {
          this.bcsNgListOptions = res.body || [];
          this.dataSource.data = this.bcsNgListOptions;
          this.dataSource.sort = this.matSorter;
          if (!this.isScully) {
            this.dataSource.paginator = this.paginator;
            this.dataSource.paginator._intl.itemsPerPageLabel = "Items par page :";
          }
          this.load = false;
        },
        err => console.error(err),
      );
  }

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(data => {
      this.loadPage();
    });
    this.registerChangeInBcsNgListOptions();
  }

  ngOnDestroy(): void {
    if (this.eventSubscriber) {
      this.eventManager.destroy(this.eventSubscriber);
    }
  }

  trackId(index: number, item: IBcsNgListOption): string {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
    return item.id!;
  }

  registerChangeInBcsNgListOptions(): void {
    this.eventSubscriber = this.eventManager.subscribe('bcsNgListOptionListModification', () => this.loadPage());
  }

  defineParamsDialog(modalRef, id){
    modalRef.componentInstance.id = id;
    modalRef.componentInstance.entityMsg = "ce choix";
    modalRef.componentInstance.serviceEntity = this.bcsNgListOptionService;
    modalRef.componentInstance.eventManagerName = "bcsNgListOptionListModification";
  }

  delete(listOption: IBcsNgListOption): void {
    const modalRef = this.modalService.open(DialogDeleteComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, listOption.id);
  }

  erase(listOption: IBcsNgListOption): void {
    const modalRef = this.modalService.open(DialogEraseComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, listOption.id);
  }

  reborn(listOption: IBcsNgListOption): void {
    const modalRef = this.modalService.open(DialogRebornComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, listOption.id);
  }

  update(listOption: IBcsNgListOption | undefined): void {
    const modalRef = this.modalService.open(BcsNgListOptionUpdateComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.listOption = listOption;
  }

  eraseAll(): void {
    if (confirm('Voulez-vous supprimer tous les choix supprimés ?')) {
      this.bcsNgListOptions?.forEach(listOption => {
        if (listOption.id !== undefined && listOption.deleted) {
          this.bcsNgListOptionService.erase(listOption.id).subscribe(() => {
            this.eventManager.broadcast('bcsNgListOptionListModification');
          });
        }
      })
    }
  }

  export(): void {
    const listOptionsExcel: IBcsNgListOptionExcel[] = [];
    this.bcsNgListOptions?.forEach(listOption => {
      listOptionsExcel.push({
        ID: listOption.id,
        NOM: listOption.name,
        LABEL: listOption.label,
        TYPE: this.listOptionTypeLabelsAndValues[listOption.type]
      });
    });
    this.excelService.exportAsExcelFile(listOptionsExcel, 'Choix de listes BCS-NG');
  }

  applyFilter(event: Event): void {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();

    if (this.dataSource?.paginator) {
      this.dataSource.paginator.firstPage();
    }
  }
}
