import { AfterContentChecked, ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { AccountService } from 'src/app/core/auth/account.service';
import { DomaineService } from 'src/app/entities/domaine/domaine.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { IDomaine } from 'src/app/shared/model/entity/domaine.model';
import { Baseline, RpsValidationScope } from '../../model/enumeration/baseline.enum';
import { IMilestone } from '../../model/milestone.model';
import { IOperation } from '../../model/operation.model';
import { OperationService } from '../table/operation/operation.service';
import { BcsNgHistoryService } from './bcs-ng-history.service';

@Component({
  selector: 'jhi-bcs-ng-historic-scope-update',
  templateUrl: './bcs-ng-historic-scope-update.component.html',
})
export class BcsNgHistoricScopeUpdateComponent extends BaseComponent implements OnInit, AfterContentChecked {
  id: string;
  validationScope: RpsValidationScope;
  newBaselineValue: Baseline;

  load: boolean = true;
  operation: IOperation;
  domaine: IDomaine;

  title = "Modifier l'historique des jalons";
  content = '';

  constructor(
    public activeModal: NgbActiveModal,
    protected historicService: BcsNgHistoryService,
    protected operationService: OperationService,
    protected accountService: AccountService,
    protected bmcsBaseService: BmcsBaseService,
    protected domaineService: DomaineService,
    private cdRef: ChangeDetectorRef,
  ) {
    super(accountService)
  }

  ngOnInit(): void {
    this.cdRef.detach();
    this.loadEntity();
  }

  ngAfterContentChecked(): void {
    this.cdRef.detectChanges();
  }

  loadEntity(): void {
    if (this.validationScope === RpsValidationScope.OPERATION) {
      this.operationService.find(this.id).subscribe(res => {
        this.operation = res.body;
        this.title += ` de l'opération ${res.body?.name}`;
        if (this.newBaselineValue === Baseline.VISIBLE_CLIENT) {
          this.content = `Le dernier historique de chaque jalon de l'opération ${res.body?.name} va passer à "Visible client".`;
        }
        if (this.newBaselineValue === Baseline.RPS_A_VENIR) {
          this.content = `Le dernier historique de chaque jalon de l'opération ${res.body?.name} va passer à "RPS à venir".`;
        }
        if (this.newBaselineValue === Baseline.RPS) {
          this.content = `Le dernier historique "RPS à venir" (s'il existe) de chaque jalon de l'opération ${res.body?.name} va passer à "RPS passée".`;
        }
        this.load = false;
      });
    } else if (this.validationScope === RpsValidationScope.DOMAINE) {
      this.domaineService.find(this.id).subscribe(res => {
        this.domaine = res.body;
        this.title += ` du domaine ${res.body?.label}`;
        if (this.newBaselineValue === Baseline.VISIBLE_CLIENT) {
          this.content = `Le dernier historique de chaque jalon du domaine ${res.body?.label} va passer à "Visible client".`;
        }
        if (this.newBaselineValue === Baseline.RPS_A_VENIR) {
          this.content = `Le dernier historique de chaque jalon du domaine ${res.body?.label} va passer à "RPS à venir".`;
        }
        if (this.newBaselineValue === Baseline.RPS) {
          this.content = `Le dernier historique "RPS à venir" (s'il existe) de chaque jalon du domaine ${res.body?.label} va passer à "RPS passée".`;
        }
        this.load = false;
      });
    }
  }

  async loadMilestones(): Promise<IMilestone[]> {
    if (this.validationScope === RpsValidationScope.OPERATION) {
      return await this.loadMilestonesForOperation(this.id);
    } else if (this.validationScope === RpsValidationScope.DOMAINE) {
      return await this.loadMilestonesForDomaine(this.id);
    } else {
      return [];
    }
  }

  async loadMilestonesForOperation(id: string): Promise<IMilestone[]> {
    return new Promise(resolve => {
      this.operationService.find(id).subscribe(res => {
        const milestones = res.body.milestones.map(m => {
          return {
            ...m,
            operation: res.body,
          }
        });
        resolve(milestones);
      });
    });
  }

  async loadMilestonesForDomaine(id: string): Promise<IMilestone[]> {
    return new Promise(resolve => {
      const domaine = this.bmcsBaseService.findDomaine(id);
      const allVersionedSystems = this.bmcsBaseService.getVersionedSystems;
      const domaineVersionedSystems = allVersionedSystems.filter(vs => domaine.systemsId?.includes(vs.system.id));
      const domaineVersionedSystemIds = domaineVersionedSystems.map(vs => vs.id);
      this.operationService.query(true).subscribe(res => {
        const domaineOperations = res.body.filter(o => domaineVersionedSystemIds.includes(o.versionedSystemId.toLocaleUpperCase()));
        const milestones: IMilestone[] = [];
        domaineOperations.forEach(o => o.milestones.forEach(m => milestones.push({ ...m, operation: o })));
        resolve(milestones);
      });
    });
  }

  async save(): Promise<void> {
    this.load = true;
    const milestonesToValidate = await this.loadMilestones();
    const milestoneIds = milestonesToValidate.map(m => m.id);
    this.historicService.updateHistoricBaseline(milestoneIds, this.newBaselineValue).subscribe(() => {
      this.load = false;
      this.activeModal.dismiss();
    }, err => {
      if (err) console.error(err);
    });
  }

  cancel(): void {
    this.activeModal.dismiss();
  }
}