import { HttpClient, HttpResponse } from '@angular/common/http';
import { Injectable } from '@angular/core';
import * as moment from 'moment';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { SERVER_BCS_URL } from 'src/app/app.constants';
import { Baseline } from '../../model/enumeration/baseline.enum';
import { IHistoric } from '../../model/historic.model';

type EntityResponseType = HttpResponse<IHistoric>;
type EntityArrayResponseType = HttpResponse<IHistoric[]>;

@Injectable({ providedIn: 'root' })
export class BcsNgHistoryService {
    public resourceUrl = SERVER_BCS_URL + 'api/historics';

    constructor(protected http: HttpClient) { }

    findByEntity(id: string): Observable<EntityArrayResponseType> {
        return this.http
            .get<IHistoric[]>(`${this.resourceUrl}/entity/${id}`, { observe: 'response' })
            .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)))
    }

    findLatestByEntity(id: string, baseline?: Baseline, noException?: boolean): Observable<EntityResponseType> {
        let url = `${this.resourceUrl}/latest/${id}`;
        if (baseline || noException) url += '?'
        if (baseline) url += `baseline=${baseline}`;
        if (noException) {
            if (baseline) url += '&';
            url += `noException=${noException}`;
        }
        return this.http
            .get<IHistoric>(url, { observe: 'response' })
            .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
    }

    findLatestByEntityForMany(idList: string[], baseline?: Baseline): Observable<EntityArrayResponseType> {
        let url = `${this.resourceUrl}/latest/many`;
        if (baseline) url += `?baseline=${baseline}`;
        return this.http
            .post<IHistoric[]>(url, idList, { observe: 'response' })
            .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
    }

    queryByBaseline(idTaskableEntity: string, baseline: Baseline): Observable<HttpResponse<any[]>> {
        return this.http
            .get<any[]>(`${this.resourceUrl}/getEntityByBaseline/${idTaskableEntity}/${baseline}`, { observe: 'response' });
    }

    queryOperationAndMilestoneByBaseline(baseline: Baseline, idOperation?: string): Observable<HttpResponse<any[]>> {
        let url = `${this.resourceUrl}/getOperationAndMilestoneByBaseline/${baseline}`;
        if (idOperation) url += `/${idOperation}`;
        return this.http.get<any[]>(url, { observe: 'response' });
    }

    update(historic: IHistoric): Observable<EntityResponseType> {
        const copy = this.convertDateFromClient(historic);
        return this.http
            .put<IHistoric>(this.resourceUrl, copy, { observe: 'response' })
            .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
    }

    updateMany(historics: IHistoric[]): Observable<EntityArrayResponseType> {
        return this.http
            .put<IHistoric[]>(this.resourceUrl + '/many', historics, { observe: 'response' })
            .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
    }

    updateHistoricBaseline(idList: string[], newBaseline: Baseline) {
        return this.http
            .put<IHistoric[]>(this.resourceUrl + '/update/baseline', { ids: idList, newBaseline: newBaseline }, { observe: 'response' })
            .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
    }

    delete(id: string): Observable<HttpResponse<{}>> {
        return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
    }

    erase(id: string): Observable<HttpResponse<{}>> {
        return this.http.delete(`${this.resourceUrl}/erase/${id}`, { observe: 'response' });
    }

    protected convertDateFromClient(historic: IHistoric): IHistoric {
        const copy = Object.assign({}, historic, {
            createdDate: historic.createdDate
        });
        return copy
    }

    protected convertDateFromServer(res: EntityResponseType): EntityResponseType {
        if (res.body) {
            res.body.createdDate = res.body.createdDate ? moment(res.body.createdDate) : undefined;
        }
        return res;
    }

    protected convertDateArrayFromServer(res: EntityArrayResponseType): EntityArrayResponseType {
        if (res.body) {
            res.body.forEach((historic: IHistoric) => {
                historic.createdDate = historic.createdDate ? moment(historic.createdDate) : undefined;
            });
        }
        return res;
    }
}