import { Scheduler } from '@bryntum/scheduler';
import { IBryntumEventClone } from 'src/app/bcs_ng/model/bryntum/bryntum-event-clone.model';
import { BryntumRecordState, IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';
import { IDependency } from 'src/app/bcs_ng/model/dependency.model';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { Utils } from 'src/app/bcs_ng/utils/util';
import { ConvertData } from './convert-data';

export class AfterTaskUpdateHandler {
    scheduler: Scheduler;
    recordStateMap: { [key: string]: IBryntumRecordState }
    clonesMap: { [key: string]: IBryntumEventClone }
    dependencyStateMap: { [key: string]: IDependency }

    constructor() {}

    /** Updates data in the scheduler after the corresponding record has been updated in the record map */
    updateDisplayElement(id: string) {
        const record = this.recordStateMap[id];
        const recordEntity = new BryntumRecordState(record.id, record.toErase, record.isUpdated, record.resource, record.event);
        const overridableData = recordEntity.getOverridableData;
        const overridden = record.resource.name !== null ||
            record.event?.name !== null ||
            record.event?.startDate !== undefined;
    
        const taskToUpdate = this.scheduler.resourceStore.getById(id);
        taskToUpdate.set('name', overridableData.name);

        const eventToUpdate = this.scheduler.eventStore.getById(id);
        const eventsToUpdate = [eventToUpdate];
        // If a clone exist, it also have to be updated
        const clone = Object.values(this.clonesMap).find(c => c.eventId === id);
        if (clone) {
            const cloneEventToUpdate = this.scheduler.eventStore.getById(clone.id);
            if (cloneEventToUpdate) {
                eventsToUpdate.push(cloneEventToUpdate);
                this.clonesMap[clone.id].event = 
                {
                    ...clone.event,
                    name: overridableData.shortName,
                    startDate: overridableData.startDate,
                    endDate: overridableData.endDate,
                }
            };
        }

        eventsToUpdate.forEach(e => {
            if (TaskableEntityTypeService.isTaskFromBmcs(record.resource.typeId)) {
                overridableData.shortName ? e.set('name', overridableData.shortName) : e.set('name', overridableData.name);
            } else {
                e.set('name', overridableData.shortName);
            }
            if (Utils.isDateValid(overridableData.startDate)) e.set('startDate', overridableData.startDate);
            if (Utils.isDateValid(overridableData.endDate)) e.set('endDate', overridableData.endDate);
        });
    
        let newCls = 'non-overridden-task';
        if (!record.resource.object) newCls = 'independant-task';
        if (overridden && record.resource.object) newCls = 'overridden-task';
        taskToUpdate.set('cls', newCls);
    }

    private updateMilestoneRecordDate(record: IBryntumRecordState, dateShift: number): void {
        const recordEntity = new BryntumRecordState(record.id, record.toErase, record.isUpdated, record.resource, record.event);
        const overriddenDate = recordEntity.getOverridableData.startDate;
        const newDate = new Date(overriddenDate.getFullYear(), overriddenDate.getMonth(), overriddenDate.getDate() + dateShift);
        record.event.startDate = newDate;
        record.event.endDate = newDate;
        const eventToUpdate = this.scheduler.eventStore.getById(record.id);
        if (Utils.isDateValid(newDate)) eventToUpdate.set('startDate', newDate);
        if (Utils.isDateValid(newDate)) eventToUpdate.set('endDate', newDate);
        const overridden = record.resource.name !== null ||
            record.event?.name !== null ||
            record.event?.startDate !== undefined;
        const taskToUpdate = this.scheduler.resourceStore.getById(record.id);
        taskToUpdate.set('cls', overridden ? 'overridden-task' : 'non-overridden-task');
        record.isUpdated = true;
    }

    /** Recursively updades the date of successors */
    updateSuccessors(id: string, dateShift: number, datesAlreadyUpdated: boolean, alreadyUpdatedRecordIds: string[]) {
        const record = this.recordStateMap[id];
    
        // The dates of the root updated element are already updated
        if (!datesAlreadyUpdated) {
            this.updateMilestoneRecordDate(record, dateShift);
            alreadyUpdatedRecordIds.push(record.id);
        }

        const successorIds = Object
            .values(this.dependencyStateMap)
            .filter(dep => dep.predecessor.id === record.resource.object?.id)
            .map(dep => dep.successor.id)
        const successorRecords = Object.values(successorIds).reduce((total: IBryntumRecordState[], current) => {
            const currentSuccessorRecords = Object
                .values(this.recordStateMap)
                .filter(potentialSuccessor => potentialSuccessor.resource.object?.id === current);
            currentSuccessorRecords.forEach(successorRecord => total.push(successorRecord));
            return total;
        }, []);
        successorRecords.forEach(r => {
            // We must determine if the current record is winning against other predecessors of the successor, with the successor updated dates
            const recordsMapCopy = JSON.parse(JSON.stringify(this.recordStateMap));
            const successorRecordCopy = JSON.parse(JSON.stringify(r));
            const successorRecordEntity = new BryntumRecordState(
                successorRecordCopy.id,
                successorRecordCopy.toErase,
                successorRecordCopy.isUpdated,
                successorRecordCopy.resource,
                successorRecordCopy.event
            );
            const successorOverriddenDate = new Date(successorRecordEntity.getOverridableData.startDate);
            const successorNewDate = new Date(
                successorOverriddenDate.getFullYear(),
                successorOverriddenDate.getMonth(),
                successorOverriddenDate.getDate() + dateShift
            );
            
            successorRecordCopy.event.startDate = successorNewDate;
            successorRecordCopy.event.endDate = successorNewDate;
            recordsMapCopy[successorRecordCopy.id] = successorRecordCopy;
        
            const maxPred = ConvertData.maxPredecessorDependency(successorRecordCopy, recordsMapCopy, this.dependencyStateMap);
            if (maxPred && maxPred.predecessor.id === record.resource.object?.id) {
                this.updateSuccessors(r.id, dateShift, false, alreadyUpdatedRecordIds);
            }
        });
    }

    /** Update duplicates of the record*/
    updateDuplicates(record: IBryntumRecordState, dateShift: number): void {
        // Determine the duplicates of the record
        const duplicates = Object.values(this.recordStateMap).filter(possibleDuplicate => {
            return record.resource.object &&
                record.resource.object?.id === possibleDuplicate.resource.object?.id &&
                record.id !== possibleDuplicate.id;
        });
        duplicates.forEach(duplicate => this.updateMilestoneRecordDate(duplicate, dateShift));
    }
}