import * as moment from 'moment';
import { IBryntumAssignment } from 'src/app/bcs_ng/model/bryntum/bryntum-assignment.model';
import { IBryntumDependency } from 'src/app/bcs_ng/model/bryntum/bryntum-dependency.model';
import { DelayType, IBryntumDelay } from 'src/app/bcs_ng/model/bryntum/bryntum-delay.model';
import { IBryntumEvent } from 'src/app/bcs_ng/model/bryntum/bryntum-event.model';
import { BryntumRecordState, IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';
import { IBryntumResource } from 'src/app/bcs_ng/model/bryntum/bryntum-resource.model';
import { DisplayTask, IDisplayTask } from 'src/app/bcs_ng/model/display-task.model';
import { IDependency } from 'src/app/bcs_ng/model/dependency.model';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { Utils } from 'src/app/bcs_ng/utils/util';
import { DisplayConfigBryntum } from './display-config';
import { IBryntumEventClone } from 'src/app/bcs_ng/model/bryntum/bryntum-event-clone.model';
import { IRoadMapBcs } from 'src/app/bcs_ng/model/road-map-bcs.model';
import { ITask } from 'src/app/bcs_ng/model/task/Itask.model';

/**
 * The tasks data is processed through three layers
 * DisplayTask: the data fetched from server
 * BryntumRecordState: allow to track the state of each element in the roadmap, so that we can save the new/updated/deleted data when we save the roadmap
 * BryntumResource, BryntumEvent, BryntumAssignment, BryntumDependencies: allow to display the data on Bryntum Scheduler
 * 
 * The merge between default values and overriding valuesonly occurs when building Bryntum Scheduler data
 */
export class ConvertData {

    constructor() { }

    /** Milestones objects are needed to proerly build dependencies */
    static getMilestones(roadMap: IRoadMapBcs): ITask[] {
        const milestones: ITask[] = [];
        roadMap?.tasks?.forEach(t => {
            if (TaskableEntityTypeService.isMilestone(t.type?.id)) {
                milestones.push(t.object);
            }
        });
        return milestones;
    }

    /** An array of display tasks is not flexible enough to register real time modifications, so we turn it into a JSON map */
    static displayTasksToRecordsMap(displayTasks: IDisplayTask[], readOnly?: boolean, editMode?: boolean): { [key: string]: BryntumRecordState } {
        const recordsMap = displayTasks.reduce((result: { [key: string]: BryntumRecordState }, displayTask: IDisplayTask) => {
            const emptyDisplayTask = new DisplayTask();
            emptyDisplayTask.additionalProperties = displayTask.additionalProperties;
            const additionalProperties = emptyDisplayTask.getAdditionalProperties;
            // If the display task is one of an operation, we must calculate the dates based on those of its milestones
            let startDate = displayTask.startDate ? new Date(displayTask.startDate?.toString()) : undefined;
            let endDate = displayTask.endDate ? new Date(displayTask.endDate?.toString()) : undefined;
            const overridden = displayTask.object && (
                displayTask.name?.length > 0 ||
                displayTask.shortName?.length > 0 ||
                displayTask.startDate !== null ||
                displayTask.endDate !== null
            );

            let eventCls = '';
            if (TaskableEntityTypeService.isOperation(displayTask.type?.id)) eventCls = 'operation-event';
            if (TaskableEntityTypeService.isMilestone(displayTask.type?.id)) eventCls = 'milestone-event';

            let taskCls = 'non-overridden-task';
            if (editMode) {
                if (!displayTask.object) {
                    taskCls = 'independant-task';
                } else if (overridden) {
                    taskCls = 'overridden-task';
                }
            }

            const displayConfTaskProperties = DisplayConfigBryntum.getDisplayConfTaskPropertiesFromBcs(displayTask.displayConfig);
            const record = new BryntumRecordState(
                displayTask.id,
                false,
                false,
                {
                    id: displayTask.id,
                    name: displayTask.name,
                    hidden: additionalProperties.hidden,
                    deleted: displayTask.deleted,
                    displayed: !additionalProperties.hidden && !displayTask.deleted,
                    typeId: displayTask.type?.id ?? null,
                    parentId: displayTask.parentId,
                    parentIndex: additionalProperties.parentIndex,
                    createdBy: displayTask.createdBy,
                    object: displayTask.object,
                    rpsObjects: displayTask.rpsObjects,
                    expanded: additionalProperties.expanded,
                    readOnly: false,
                    cls: taskCls,
                    independantMilestone: additionalProperties.independantMilestone ?? false,
                    fontColor: displayConfTaskProperties?.resourceFontColor,
                    fontSize: displayConfTaskProperties?.resourceFontSize,
                    rowHeight: displayConfTaskProperties?.rowHeight,
                },
                {
                    id: displayTask.id,
                    name: displayTask.shortName,
                    locationId: additionalProperties.resourcesId,
                    startDate,
                    endDate,
                    readOnly: readOnly ? readOnly : false,
                    parentId: displayTask.parentId,
                    object: displayTask.object,
                    type: displayTask.type ?? null,
                    hidePredecessors: additionalProperties.hidePredecessors,
                    cls: eventCls,
                    delayEvent: false,
                    displayConfig: displayTask.displayConfig,
                    iconCls: displayConfTaskProperties?.shape,
                    eventColor: displayConfTaskProperties?.bgColor,
                    label: displayConfTaskProperties?.position,
                    offsetMultiColorEventConfig: displayConfTaskProperties?.multiColorsEventConfigs,
                    fontSize: displayConfTaskProperties?.eventFontSize,
                    rowHeight: displayConfTaskProperties?.rowHeight,
                    independantMilestone: additionalProperties.independantMilestone ?? false,
                }
            );
            result[displayTask.id] = record;
            return result;
        }, {});

        return recordsMap;
    }

    /** We do the same for dependencies as for display tasks. This allows to register real time modifications of time gaps */
    static displayTasksToDependenciesMap(displayTasks: IDisplayTask[]): { [key: string]: IDependency } {
        const dependencyRecordsMap = {};
        displayTasks.forEach(displayTask => {
            displayTask.object?.predecessorDependencies?.forEach(dep => {
                dependencyRecordsMap[dep.id] = { ...dep };
            });
        });
        return dependencyRecordsMap;
    }

    static recordsToClonesMap(recordsMap: { [key: string]: IBryntumRecordState }): { [key: string]: IBryntumEventClone } {
        return Object.values(recordsMap).reduce((total: { [key: string]: IBryntumEventClone }, record) => {
            if (record.event.locationId && record.event.locationId?.length > 0) {
                const cloneId = Utils.createUUID();
                const recordEntity = new BryntumRecordState(record.id, record.toErase, record.isUpdated, record.resource, record.event);
                const overridableData = recordEntity.getOverridableData;
                total[cloneId] = {
                    id: cloneId,
                    eventId: record.id,
                    event: {
                        ...record.event,
                        id: cloneId,
                        name: overridableData.shortName,
                        startDate: overridableData.startDate,
                        endDate: overridableData.endDate,
                        cls: record.event?.cls + ' clone-event',
                        locationId: record.event.locationId
                    }
                }
                record.event.cloneId = cloneId;
            }
            return total;
        }, {});
    }

    static recordsMapToDelaysMap(
        recordsMap: { [key: string]: IBryntumRecordState },
        dependenciesMap: { [key: string]: IDependency }
    ): { [key: string]: IBryntumDelay } {
        const delaysMap: { [key: string]: IBryntumDelay } = {};
        const records = Object.values(recordsMap);
        records.forEach(record => {
            if (this.isShiftedFromContract(record)) {
                const recordEventDelays = this.buildEventDelays(record, recordsMap, dependenciesMap, null);
                recordEventDelays.forEach(delayEvent => {
                    delaysMap[delayEvent.id] = delayEvent;
                });
            }
            if (this.isShiftedFromRps(record)) {
                const delayId = Utils.createUUID();
                delaysMap[delayId] = {
                    id: delayId,
                    eventId: record.id,
                    previousEventId: '',
                    type: DelayType.RPS,
                }
            }
        });
        return delaysMap;
    }

    /** Build resources, events and assignments from records */
    static recordsToResourcesAndEvents(
        recordsMap: { [key: string]: IBryntumRecordState },
        readOnly?: boolean
    ): { resources: IBryntumResource[], events: IBryntumEvent[], assignments: IBryntumAssignment[] } {
        const records = Object.values(recordsMap);

        const resources: IBryntumResource[] = [];
        const events: IBryntumEvent[] = [];
        const assignments: IBryntumAssignment[] = [];

        records.forEach(record => {
            const recordEntity = new BryntumRecordState(record.id, record.toErase, record.isUpdated, record.resource, record.event);
            const overridableData = recordEntity.getOverridableData;

            const overriddenResource: IBryntumResource = {
                ...record.resource,
                name: overridableData.name,
            };
            // We must reset the displayed attribute if the parent of a record is not displayed
            if (overriddenResource.parentId && !recordsMap[overriddenResource.parentId]?.resource.displayed) {
                overriddenResource.displayed = false;
            }
            resources.push(overriddenResource);

            const overriddenEvent: IBryntumEvent = {
                ...record.event,
                name: overridableData.shortName ?? overridableData.name,
                startDate: overridableData.startDate,
                endDate: overridableData.endDate,
                readOnly: readOnly ? readOnly : false,
            };
            events.push(overriddenEvent);

            const inlineAssignment = {
                resourceId: record.id,
                eventId: record.id,
                readOnly: readOnly ? readOnly : false
            }
            assignments.push(inlineAssignment);
        });

        return { resources, events, assignments };
    }

    /** Build clone events for events we want to display on parents or neighbours */
    static clonesMapToCloneEvents(
        clonesMap: { [key: string]: IBryntumEventClone }
    ): { cloneEvents: IBryntumEvent[], cloneAssignments: IBryntumAssignment[] } {
        const cloneEvents = [];
        const cloneAssignments: IBryntumAssignment[] = [];

        Object.values(clonesMap).forEach(eventClone => {
            cloneEvents.push({ ...eventClone.event, eventId: eventClone.eventId });
            eventClone.event.locationId.forEach(location => {
                cloneAssignments.push({ resourceId: location, eventId: eventClone.id, readOnly: true })
            })
        });

        return { cloneEvents, cloneAssignments };
    }

    /** Build dependencies from records and dependencyRecords */
    static recordsToDependencies(
        recordsMap: { [key: string]: IBryntumRecordState },
        dependenciesMap: { [key: string]: IDependency },
        milestones: ITask[],
    ): IBryntumDependency[] {
        const records = Object.values(recordsMap);
        const dependencies: IBryntumDependency[] = [];

        records.forEach(record => {
            if (TaskableEntityTypeService.isMilestone(record.resource.typeId)) {
                const predecessors = this.getPredecessors(record.resource.object.id, recordsMap, dependenciesMap);
                if (!record.event.hidePredecessors) {
                    const recordDependencies = predecessors.map(predecessor => {
                        const cls = this.milestoneCls(record?.resource?.object?.id, predecessor?.resource?.object?.id, milestones);
                        return {
                            from: predecessor.id,
                            to: record.id,
                            cls,
                        }
                    });
                    recordDependencies.forEach(dependency => dependencies.push(dependency));
                }
            }
        });

        return dependencies;
    }

    private static milestoneCls(milestoneId1: string, milestoneId2: string, milestones: ITask[]): string {
        const milestone1 = milestones.find(m => m.id === milestoneId1);
        const milestone2 = milestones.find(m => m.id === milestoneId2);
        if (!milestone1 || !milestone2) return 'intra-operation';
        return milestone1.operation.id === milestone2.operation.id ? 'intra-operation' : 'inter-operation';
    }

    static recordsMapToDelayEvents(
        recordsMap: { [key: string]: IBryntumRecordState },
        delaysMap: { [key: string]: IBryntumDelay }
    ): { delayEvents: IBryntumEvent[], delayAssignments: IBryntumAssignment[] } {
        const delayEvents: IBryntumEvent[] = [];
        const delayAssignments: IBryntumAssignment[] = [];
        const delays = Object.values(delaysMap);

        delays.forEach(delay => {
            const delayRecord = recordsMap[delay.eventId];
            const delayEvent = delay.type === DelayType.CONTRACT ?
                this.buildContractDelayEvent(delay, recordsMap, delaysMap) :
                this.buildRpsDelayEvent(delay, recordsMap);
            delayEvents.push(delayEvent);
            const delayAssignment = {
                resourceId: delay.eventId,
                eventId: delay.id,
                readOnly: true,
            }
            delayAssignments.push(delayAssignment);

            if (delayRecord.event.locationId) {
                delayRecord.event.locationId.forEach(location => {
                    const delayAssignment = {
                        resourceId: location,
                        eventId: delay.id,
                        readOnly: true,
                    }
                    delayAssignments.push(delayAssignment);
                });
            }
        });

        return { delayEvents, delayAssignments };
    }

    /** From the records, we build arrays that can be directly injected in the scheduler stores */
    static recordsToMainData(
        recordsMap: { [key: string]: IBryntumRecordState },
        dependenciesMap: { [key: string]: IDependency },
        milestones: ITask[],
        readOnly?: boolean
    ): {
        resources: IBryntumResource[],
        events: IBryntumEvent[],
        assignments: IBryntumAssignment[],
        dependencies: IBryntumDependency[]
    } {
        const { resources, events, assignments } = ConvertData.recordsToResourcesAndEvents(recordsMap, readOnly);
        const dependencies = ConvertData.recordsToDependencies(recordsMap, dependenciesMap, milestones);
        return { resources, events, assignments, dependencies };
    }

    /** When saving the roadmap, the records map must be converted into an array of Display Task entities */
    static recordsToDisplayTasks(records: IBryntumRecordState[], roadMapId: string): IDisplayTask[] {
        return records.map(record => {
            let startDate = record.event.startDate ? moment(record.event.startDate) : null;
            let endDate = record.event.endDate ? moment(record.event.endDate) : null;
            if (!TaskableEntityTypeService.isMilestone(record.resource.typeId) && record.resource.object) {
                startDate = null;
                endDate = null;
            }
            let name = null;
            if (!record.resource.object) {
                name = record.resource.name;
            } else if (TaskableEntityTypeService.isDomaine(record.resource.typeId)) {
                name = record.resource.name ?? record.resource.object?.domaine; // Domaine entity has no "name" field
            } else {
                name = record.resource.name ?? record.resource.object?.name;
            }

            const displayTask: IDisplayTask = {
                id: record.id,
                name: record.resource.name,
                shortName: record.event.name,
                type: record.resource.object ? { id: record.resource.typeId } as any : undefined,
                roadMap: { id: roadMapId },
                parentId: record.resource.parentId,
                createdBy: record.resource.createdBy,
                object: { id: record?.resource?.object?.id } as any,
                deleted: record.resource.deleted,
                startDate,
                endDate,
                additionalProperties: JSON.stringify({
                    hidden: record.resource.hidden,
                    hidePredecessors: record.event.hidePredecessors,
                    expanded: record.resource.expanded,
                    parentIndex: record.resource.parentIndex,
                    independantMilestone: record.resource.independantMilestone,
                    resourcesId: record.event.locationId
                }),
                displayConfig: record.event.displayConfig,
            }
            return displayTask;
        });
    }

    /** Id is the id of the unerlying milestone */
    static getPredecessors(
        id: string,
        recordsMap: { [key: string]: IBryntumRecordState },
        dependenciesMap: { [key: string]: IDependency }
    ): IBryntumRecordState[] {
        return Object
            .values(dependenciesMap)
            .filter(dep => dep.successor.id === id)
            .reduce((total, dep) => {
                const depRecords = Object.values(recordsMap).filter(rec => rec?.resource?.object?.id === dep?.predecessor?.id);
                depRecords?.forEach(record => total.push(record));
                return total;
            }, []);
    }

    /** Id is the id of the unerlying milestone */
    static getPredecessorDependencies(id: string, dependenciesMap: { [key: string]: IDependency }): IDependency[] {
        return Object.values(dependenciesMap).filter(dep => dep.successor.id === id)
    }

    /** Returns true if the milestone display task doesn't have the same date as its object contract date */
    private static isShiftedFromContract(record: IBryntumRecordState): boolean {
        if (!record.resource.object?.contractDate) return false;
        const recordEntity = new BryntumRecordState(record.id, record.toErase, record.isUpdated, record.resource, record.event);
        const overridableData = recordEntity.getOverridableData;
        const recordObjectContractDateWithTime = new Date(record.resource.object.contractDate?.toString());
        const overridableDataStartDate = new Date(overridableData.startDate.toDateString());
        const recordObjectContractDate = new Date(recordObjectContractDateWithTime.toDateString());
        return overridableDataStartDate?.getTime() > recordObjectContractDate?.getTime();
    }

    /** Returns true if the underlying milestone has an RPS historic
     * and if the record current date is different from the date saved in this historic
    */
    private static isShiftedFromRps(record: IBryntumRecordState): boolean {
        const rpsObjects = record.resource.rpsObjects;
        if (!rpsObjects?.length) return false;
        if (rpsObjects.length === 1) {
            const recordEntity = new BryntumRecordState(record.id, record.toErase, record.isUpdated, record.resource, record.event);
            const overridableData = recordEntity.getOverridableData;
            const recordDate = overridableData.startDate;
            const rpsDate = new Date(rpsObjects[0].startDate?.toString());
            return recordDate.getTime() > rpsDate.getTime();
        }
        if (rpsObjects.length === 2) {
            const latestRpsDate = new Date(rpsObjects[1].startDate?.toString());
            const penultimateRpsDate = new Date(rpsObjects[0].startDate?.toString());
            return latestRpsDate.getTime() > penultimateRpsDate.getTime();
        }
    }

    /** A - max(predDate + predDependencyTimeGap)
     * Since it impacts the display, we must define a deterministic way to settle between two equal (predDate + predDependencyTimeGap)
     * B - The date has more importance than the timeGap
     * C - If both the date and the timeGap are equal, then we select the most recently created
     * D - In last resort, we take the highest id alphabetically
    */
    static maxPredecessorDependency(
        record: IBryntumRecordState,
        recordsMap: { [key: string]: IBryntumRecordState },
        dependenciesMap: { [key: string]: IDependency }
    ): IDependency {
        const predecessorDependencies = ConvertData.getPredecessorDependencies(record.resource.object.id, dependenciesMap);
        let maxPredDep: IDependency = { ...predecessorDependencies[0] };
        predecessorDependencies.slice(1).forEach(dep => {
            const predRecord = Object.values(recordsMap).find(r => r.resource.object?.id === dep.predecessor?.id);
            if (predRecord) {
                const predRecordEntity = new BryntumRecordState(
                    predRecord.id,
                    predRecord.toErase,
                    predRecord.isUpdated,
                    predRecord.resource,
                    predRecord.event
                );
                const predOverridableData = predRecordEntity.getOverridableData;
                const predStartDate = new Date(predOverridableData.startDate);
                const projectedDate = Utils.addTimegapToDate(predStartDate, dep.timeGapDuration, dep.timeGapUnit);
                const maxPredRecord = Object.values(recordsMap).find(r => r.resource.object?.id === maxPredDep.predecessor.id);
                const maxPredStartDate = new Date(maxPredRecord.event.startDate ?? maxPredRecord.resource.object?.startDate.toString());
                const maxProjectedDate = Utils.addTimegapToDate(maxPredStartDate, maxPredDep.timeGapDuration, maxPredDep.timeGapUnit);
                if (projectedDate > maxProjectedDate) { // A
                    maxPredDep = { ...dep };
                } else if (projectedDate.getTime() === maxProjectedDate.getTime()) {
                    if (predStartDate > maxPredStartDate) { // B
                        maxPredDep = { ...dep };
                    } else if (predStartDate === maxPredStartDate) {
                        const predCreationDate = new Date(predRecord.resource.object.createdDate.toString());
                        const maxPredCreationDate = new Date(maxPredRecord.resource.object.createdDate.toString())
                        if (predCreationDate > maxPredCreationDate) { // C
                            maxPredDep = { ...dep };
                        } else {
                            if (dep.id > maxPredDep.id) { // D
                                maxPredDep = { ...dep };
                            }
                        }
                    }
                }
            }
        });
        return maxPredDep;
    }

    private static buildEventDelays(
        record: IBryntumRecordState,
        recordsMap: { [key: string]: IBryntumRecordState },
        dependenciesMap: { [key: string]: IDependency },
        previousEventId: string,
    ): IBryntumDelay[] {
        const records = Object.values(recordsMap);
        const eventDelays: IBryntumDelay[] = [];
        const delayId = Utils.createUUID();
        const eventDelay = {
            id: delayId,
            eventId: record.id,
            previousEventId,
            type: DelayType.CONTRACT,
        };
        eventDelays.push(eventDelay);

        // Among all the successors of the records, determine those that will be impacted by the delay
        record.resource.object?.successorDependencies?.forEach(successorDep => {
            const successorRecord = records.find(record => record.resource.object?.id === successorDep.successor.id);
            if (successorRecord) {
                const maxPredDep = this.maxPredecessorDependency(successorRecord, recordsMap, dependenciesMap);
                if (maxPredDep && maxPredDep.predecessor && maxPredDep.predecessor.id === record.resource.object?.id) {
                    const successorEventDelays = this.buildEventDelays(successorRecord, recordsMap, dependenciesMap, record.event.id);
                    successorEventDelays.forEach(successorEvDelay => eventDelays.push(successorEvDelay));
                }
            }
        });

        return eventDelays;
    }

    static getOriginalDelayedEvent(
        delay: IBryntumDelay,
        recordsMap: { [key: string]: IBryntumRecordState },
        delaysMap: { [key: string]: IBryntumDelay }
    ): IBryntumEvent {
        if (!delay.previousEventId) {
            return recordsMap[delay.eventId].event;
        }
        else {
            const delays = Object.values(delaysMap);
            const previousDelay = delays.find(del => del.eventId === delay.previousEventId);
            return this.getOriginalDelayedEvent(previousDelay, recordsMap, delaysMap);
        }
    }

    static buildContractDelayEvent(
        delay: IBryntumDelay,
        recordsMap: { [key: string]: IBryntumRecordState },
        delaysMap: { [key: string]: IBryntumDelay }
    ): IBryntumEvent {
        const originalEvent = this.getOriginalDelayedEvent(delay, recordsMap, delaysMap);
        const delayRecord = recordsMap[delay.eventId];
        const originalDelayRecord = recordsMap[originalEvent.id];

        const originalEventContractDate = new Date(originalDelayRecord.resource.object?.contractDate?.toString());
        const originalEventStartDate = originalEvent.startDate ??
            new Date(originalDelayRecord.resource.object?.startDate?.toString());
        const name = delay.previousEventId ?
            (originalEvent.name ?? originalDelayRecord.resource.object?.shortName) :
            "";
        const delayDuration = (originalEventStartDate.getTime() - originalEventContractDate.getTime()) / (1000 * 3600 * 24);
        const delayEndDate = delayRecord.event.startDate ?? new Date(delayRecord.resource.object?.startDate?.toString());
        const delayStartDate = new Date(delayEndDate);
        delayStartDate.setDate(delayStartDate.getDate() - delayDuration);
        const properties = DisplayConfigBryntum.getDisplayConfTaskPropertiesFromBcs(delayRecord.event?.displayConfig);
        const delayEvent = {
            id: delay.id,
            name,
            locationId: delayRecord.event.locationId,
            startDate: delayStartDate,
            endDate: delayEndDate,
            readOnly: true,
            object: null,
            type: null,
            parentId: delayRecord.resource?.parentId,
            hidePredecessors: false,
            cls: "b-sch-event-delay-arrow",
            delayEvent: true,
            displayTask: delayRecord.event,
            delayArrowConf: properties?.delayArrowConf
        };
        return delayEvent;
    }

    static buildRpsDelayEvent(
        delay: IBryntumDelay,
        recordsMap: { [key: string]: IBryntumRecordState },
    ): IBryntumEvent {
        const delayRecord = recordsMap[delay.eventId];
        const rpsObjects = delayRecord.resource.rpsObjects;
        const delayStartDate = new Date(rpsObjects[0].startDate?.toString());
        let delayEndDate = new Date();
        if (rpsObjects.length === 1) {
            delayEndDate = delayRecord.event.startDate ?? new Date(delayRecord.resource.object?.startDate.toString());
        } else {
            delayEndDate = new Date(rpsObjects[1].startDate?.toString());
        }
        const properties = DisplayConfigBryntum.getDisplayConfTaskPropertiesFromBcs(delayRecord.event?.displayConfig);
        const delayEvent = {
            id: delay.id,
            name: '',
            locationId: delayRecord.event.locationId,
            startDate: delayStartDate,
            endDate: delayEndDate,
            readOnly: true,
            object: null,
            type: null,
            parentId: delayRecord.resource?.parentId,
            hidePredecessors: false,
            cls: "b-sch-event-delay-arrow-rps",
            delayEvent: true,
            displayTask: delayRecord.event,
            delayRpsArrowConf: properties?.delayRpsArrowConf
        };

        return delayEvent;
    }
}