import { HttpResponse } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { DateHelper, Model, Scheduler } from '@bryntum/scheduler';
import * as moment from 'moment';
import { IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';
import { DisplayConfRoadMapProperties, FilterArrowMode, TimeRangeMode } from 'src/app/bcs_ng/model/bryntum/display-configuration/display-conf-road-map-properties';
import { DisplayConfTaskArrowProperties } from 'src/app/bcs_ng/model/bryntum/display-configuration/display-conf-task-arrow-properties';
import { DisplayConfTaskProperties } from 'src/app/bcs_ng/model/bryntum/display-configuration/display-conf-task-properties';
import { DisplayConfiguration } from 'src/app/bcs_ng/model/display-configuration.model';
import { DisplayRule, IDisplayRule } from 'src/app/bcs_ng/model/display-rule.model';
import { DisplayConfigurationType } from 'src/app/bcs_ng/model/enumeration/display-configuration-type';
import { RuleType } from 'src/app/bcs_ng/model/enumeration/rule-type.enum';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { FilterWithAccents } from 'src/app/shared/util/filter-with-accents';
import { MultiColorsEventConfig, MultiColorsEventConfigRule, OffsetMultiColorEventConfig } from '../../../model/bryntum/display-configuration/multi-colors-event-config';
import { DisplayRuleService } from '../../display-rule/display-rule.service';
import { SchedulerStoresHandler } from './scheduler-stores';

@Injectable()
export class DisplayConfigBryntum {

    scheduler?: Scheduler;
    displayConfigRules?: DisplayRule[] = [];
    charterConfigRules?: DisplayRule[] = [];
    recordStateMap: { [key: string]: IBryntumRecordState } = {};

    constructor(protected filterWithAccents: FilterWithAccents,
        protected schedulerStoresHandler: SchedulerStoresHandler,
        protected displayRuleService: DisplayRuleService
    ) {
        this.displayRuleService.findCharter().subscribe((res: HttpResponse<IDisplayRule[]>) => {
            this.setCharterConfigRules = res.body || [];
        });
    }

    public get getDisplayConfigRules() {
        return this.displayConfigRules;
    }

    public set setDisplayConfigRules(displayRules: DisplayRule[]) {
        this.displayConfigRules = displayRules;
    }

    public addDisplayConfRules(displayRules: DisplayRule[]) {
        this.displayConfigRules = this.displayConfigRules ? this.displayConfigRules.concat(displayRules) : displayRules;
    }

    public get getCharterConfigRules() {
        return this.charterConfigRules;
    }

    public set setCharterConfigRules(charterRules: DisplayRule[]) {
        this.charterConfigRules = charterRules;
    }
    public getProperties(resourceFontSize: number, eventFontSize: number,
        resourceFontColor: string, timeRangeStartDate: Date,
        timeRangeMode: TimeRangeMode,
        filterDelayArrow: FilterArrowMode,
        filterRpsDelayArrow: FilterArrowMode): string {
        let result: DisplayConfRoadMapProperties = new DisplayConfRoadMapProperties(
            this.scheduler.zoomLevel,
            this.scheduler.startDate,
            this.scheduler.endDate,
            this.scheduler.rowHeight,
            resourceFontSize,
            eventFontSize,
            resourceFontColor,
            timeRangeStartDate,
            timeRangeMode,
            filterDelayArrow,
            filterRpsDelayArrow
        );
        return JSON.stringify(result);
    }

    public applyProperties(displayConfig: DisplayConfiguration) {
        if (displayConfig) {
            let properties: DisplayConfRoadMapProperties = JSON.parse(displayConfig?.properties);
            this.scheduler.zoomLevel = properties.zoomLevel;
            this.scheduler.startDate = properties.startDate;
            this.scheduler.endDate = properties.endDate;
            this.scheduler.rowHeight = properties.rowHeight ?? 40;
            this.scheduler.barMargin = this.scheduler.rowHeight / 20;
            this.applyTextFormatting(properties.resourceFontSize, properties.resourceFontColor);
            this.applyEventTextFormatting(properties.eventFontSize);
            this.applyFilterArrow(properties?.filterDelayArrow, 'hiddeArrow');
            this.applyFilterArrow(properties?.filterRpsDelayArrow, 'hiddeRpsArrow');
            if (displayConfig.displayRules) {
                this.setDisplayConfigRules = displayConfig.displayRules;
                this.applyConfigRules();
            }
        }
    }
    public createNewDisplayConfiguration(id: string, name: string, type: DisplayConfigurationType, properties: string): DisplayConfiguration {
        return { ... new DisplayConfiguration(), id: id, name: name, type: type, properties: properties, displayRules: this.getDisplayConfigRules };
    }

    public updateDisplayConfTaskProperties(displayConfTaskProperties: DisplayConfTaskProperties,
        bgColor?: string,
        shape?: string,
        position?: string,
        multiColorsEventConfigs?: MultiColorsEventConfig[]): string {
        displayConfTaskProperties.bgColor = bgColor;
        displayConfTaskProperties.shape = shape;
        displayConfTaskProperties.position = position;
        displayConfTaskProperties.multiColorsEventConfigs = multiColorsEventConfigs;
        return JSON.stringify(displayConfTaskProperties);
    }
    public updateArrowDisplayConfTaskProperties(displayConfTaskProperties: DisplayConfTaskProperties,
        delayArrowConf?: DisplayConfTaskArrowProperties,
        delayRpsArrowConf?: DisplayConfTaskArrowProperties): string {
        displayConfTaskProperties.delayArrowConf = delayArrowConf;
        displayConfTaskProperties.delayRpsArrowConf = delayRpsArrowConf;
        return JSON.stringify(displayConfTaskProperties);
    }


    //------------------------------------------------------------------------------------------------------------
    //--                                         FILTER ARROW CONFIG                                            --
    //------------------------------------------------------------------------------------------------------------
    public applyFilterArrow(fiterArrowMode: FilterArrowMode, filterId: string) {
        if (fiterArrowMode == FilterArrowMode.DISPLAY_ALL) this.scheduler.eventStore.removeFilter(filterId);
        else if (fiterArrowMode == FilterArrowMode.HIDE_ALL) {
            const clsArrow = filterId == 'hiddeArrow' ? 'b-sch-event-delay-arrow' : 'b-sch-event-delay-arrow-rps';
            this.scheduler.eventStore.filter({ id: filterId, filterBy: record => record?.getData('cls') != clsArrow })
        }
        else {
            const delayArrowConf = filterId == 'hiddeArrow' ? 'delayArrowConf' : 'delayRpsArrowConf'
            this.scheduler.eventStore?.filter({ id: filterId, filterBy: record => record?.getData(delayArrowConf)?.hidden != true });
        }
    }



    //------------------------------------------------------------------------------------------------------------
    //--                                        RESOURCE STYLE CONFIG                                          --
    //------------------------------------------------------------------------------------------------------------
    public applyTextFormatting(fontSize: number, resourceFontColor: string): void {
        this.scheduler.resourceStore.traverse((resource: Model) => {
            const event = this.scheduler.eventStore.getById(resource.id);
            const rowHeight = resource.getData('rowHeight');
            resource.set('fontSize', fontSize);
            resource.set('fontColor', resourceFontColor);
            event.set('rowHeight', rowHeight);
        });
        Object.values(this.recordStateMap).forEach(record => {
            const resourceData = this.scheduler.resourceStore.getById(record.id);
            if (record.resource.fontSize) resourceData.set('fontSize', record.resource.fontSize);
            if (record.resource.fontColor) resourceData.set('fontColor', record.resource.fontColor);
        });
    }

    public applyIndividualTextFormatting(id: string, rowHeight: number, fontSize: number, fontColor: string): void {
        const resource = this.scheduler.resourceStore.getById(id);
        resource.set('rowHeight', rowHeight);
        resource.set('fontSize', fontSize);
        resource.set('fontColor', fontColor);
    }

    //------------------------------------------------------------------------------------------------------------
    //--                                        EVENT STYLE CONFIG                                              --
    //------------------------------------------------------------------------------------------------------------

    public applyConfigRules() {
        this.resetStyle();
        this.scheduler.eventStore.traverse((event: Model) => {
            this.applyConfigRulesByEvent(event);
        });
    }

    public applyConfigRulesByEvent(event) {
        const rulesToApply = this.charterConfigRules.concat(this.displayConfigRules);
        rulesToApply?.forEach(rule => {
            if (rule.ruleType && event.getData('type')?.id === rule.taskableEntityId) {
                let eventValueToCompare = this.getValueByColumnName(event, rule);
                if (rule.ruleType === RuleType.CONTAINS)
                    if (this.filterWithAccents.compareStrings(eventValueToCompare, rule.value, true)) {
                        if (rule.multiColorsEventConfigsRules) this.applyMultiColorsEventConfig(event, rule.multiColorsEventConfigsRules)
                        this.applyStyle(event, rule);
                    }
                    else if (this.filterWithAccents.equalsStrings(eventValueToCompare, rule.value)) {
                        this.applyStyle(event, rule);
                        if (rule.multiColorsEventConfigsRules) this.applyMultiColorsEventConfig(event, rule.multiColorsEventConfigsRules)
                    }
            } else if (event.getData('type')?.id === rule.taskableEntityId) {
                if (rule.multiColorsEventConfigsRules) this.applyMultiColorsEventConfig(event, rule.multiColorsEventConfigsRules)
                this.applyStyle(event, rule);
            }
        });
    }

    public applyStyle(event: Model, rule: DisplayRule) {
        let displayConfTaskProperties = DisplayConfigBryntum.getCurrentDisplayConfTaskProperties(event);
        if (!event.getData('cls').includes('clone-event')) this.setStyle(event, rule, displayConfTaskProperties);
        if (event.getData('cloneId')) {
            let clone = this.scheduler.eventStore.getById(event.getData('cloneId'))
            this.setStyle(clone, rule, displayConfTaskProperties);
        }
    }

    public setStyle(event: Model, rule: DisplayRule, displayConfTaskProperties: DisplayConfTaskProperties) {
        if (!displayConfTaskProperties?.bgColor) event.set('eventColor', rule.bgColor);
        if (!displayConfTaskProperties?.shape) rule.shape ? event.set('iconCls', 'b-fa fa-solid fa-' + rule.shape) : event.set('iconCls', null);
        if (!displayConfTaskProperties?.position) event.set('label', rule.position);
    }

    public resetStyle() {
        this.scheduler.eventStore.traverse((event: Model) => {
            let displayConfTaskProperties = DisplayConfigBryntum.getCurrentDisplayConfTaskProperties(event);
            if (!displayConfTaskProperties?.bgColor) event.set('eventColor', null);
            if (!displayConfTaskProperties?.shape) event.set('iconCls', null);
            if (!displayConfTaskProperties?.position) event.set('label', null);
            if (!displayConfTaskProperties?.multiColorsEventConfigs) event.set('offsetMultiColorEventConfig', null);
        });
    }

    public getValueByColumnName(event: Model, rule: DisplayRule) {
        if (rule.column === 'shortName') return event.getData('name');
        else if (rule.column === 'name') return event.getData('object')?.name;
        else if (rule.column === 'type') {
            let type;
            if (TaskableEntityTypeService.isMilestone(event.getData('type').id)) type = event.getData('object')?.typeMilestone;
            else if (TaskableEntityTypeService.isOperation(event.getData('type').id)) type = event.getData('object')?.typeOperation;
            return type?.name + type?.label;
        }
    }

    public static getDisplayConfTaskPropertiesFromBcs(displayConfig: DisplayConfiguration): DisplayConfTaskProperties {
        return displayConfig && displayConfig?.properties ? JSON.parse(displayConfig?.properties) : null;
    }

    public static getCurrentDisplayConfTaskProperties(record: Model): DisplayConfTaskProperties {
        if (record && record.getData('displayConfig')) {
            let displayConfig = DisplayConfigBryntum.getDisplayConfTaskPropertiesFromBcs(record.getData('displayConfig'));
            if (displayConfig) {
                return {
                    bgColor: displayConfig.bgColor,
                    shape: displayConfig.shape,
                    position: displayConfig.position,
                    multiColorsEventConfigs: displayConfig.multiColorsEventConfigs,
                    rowHeight: displayConfig.rowHeight,
                    resourceFontSize: displayConfig.resourceFontSize,
                    eventFontSize: displayConfig.eventFontSize,
                    resourceFontColor: displayConfig.resourceFontColor,
                    delayArrowConf: displayConfig?.delayArrowConf,
                    delayRpsArrowConf: displayConfig?.delayRpsArrowConf
                };
            }
        }
    }

    public applyEventTextFormatting(fontSize: number): void {
        this.scheduler.eventStore.traverse((event: Model) => {
            event.set('fontSize', fontSize);
        });
        Object.values(this.recordStateMap).forEach(record => {
            const eventData = this.scheduler.eventStore.getById(record.id);
            if (record.event.fontSize) eventData.set('fontSize', record.event.fontSize);
        });
    }

    //------------------------------------------------------------------------------------------------------------
    //--                                        MULTI COLOR CONFIG                                              --
    //------------------------------------------------------------------------------------------------------------

    public applyMultiColorsEventConfig(event: Model, multiColorsEventConfigRule: MultiColorsEventConfigRule) {
        //Check if event has overrided multi color event congif
        const displayConf = DisplayConfigBryntum.getCurrentDisplayConfTaskProperties(event);
        if (!(displayConf?.multiColorsEventConfigs)) this.setMultiColorsEventConfig(event, multiColorsEventConfigRule);
    }

    public setMultiColorsEventConfig(event: Model, multiColorsEventConfigRule: MultiColorsEventConfigRule) {
        let multiColorsEventConfig: MultiColorsEventConfig[] = event.getData('offsetMultiColorEventConfig') ?? [];
        const eventMilestones = SchedulerStoresHandler.getChildEventByEvent(this.scheduler, event).filter(c => TaskableEntityTypeService.isMilestone(c.getData('type')?.id));
        if (eventMilestones?.length > 0) {
            const startMilestones = eventMilestones
                .filter(c => c.getData('object')?.typeMilestone?.id === multiColorsEventConfigRule.milestoneStartTypeId)
                .sort((a, b) =>
                    new Date(a.getData('startDate')).getTime() - new Date(b.getData('startDate')).getTime()
                );
            const endMilestones = eventMilestones
                .filter(c => c.getData('object')?.typeMilestone?.id === multiColorsEventConfigRule.milestoneEndTypeId)
                .sort((a, b) =>
                    new Date(a.getData('startDate')).getTime() - new Date(b.getData('startDate')).getTime()
                );
            if (startMilestones?.length > 0 && endMilestones?.length > 0) {
                let endIdx = 0;
                for (const startM of startMilestones) {
                    for (let i = endIdx; i < endMilestones.length; i++) {
                        if (startM.getData('startDate') < endMilestones[i].getData('startDate')) {
                            multiColorsEventConfig.push({
                                ... new MultiColorsEventConfig(),
                                startOffsetEventId: startM.getData('id'),
                                endOffsetEventId: endMilestones[i].getData('id'),
                                color: multiColorsEventConfigRule.color
                            });
                            endIdx = i + 1;
                            break;
                        }
                    }
                    if (endIdx === endMilestones.length) break;
                }
            }
        }
        event.set('offsetMultiColorEventConfig', multiColorsEventConfig.length > 0 ? multiColorsEventConfig : null);
    }

    public calculateOffsetMultiColorEventConfig(event: Model, multiColorsEventConfig: MultiColorsEventConfig[]): OffsetMultiColorEventConfig[] {
        let offset: OffsetMultiColorEventConfig[] = [];
        let startDateMainEvent = moment(event.get('startDate'));

        multiColorsEventConfig.forEach(config => {
            const startEvent = this.scheduler.eventStore.getById(config.startOffsetEventId);
            const endEvent = this.scheduler.eventStore.getById(config.endOffsetEventId);

            if (startEvent && endEvent) {
                offset.push({
                    ... new OffsetMultiColorEventConfig(),
                    startOffset: this.getDistanceForDuration(startDateMainEvent, moment(startEvent.getData('startDate'))),
                    widthOffset: this.getDistanceForDuration(moment(startEvent.getData('startDate')), moment(endEvent.getData('endDate'))),
                    color: config.color + 'a3'
                });
            }
        });
        return offset;
    }

    public getDistanceBetweenDates(x, y) {
        return this.scheduler.timeAxisViewModel.getDistanceBetweenDates(x, y);
    }

    public getDistanceForDuration(x, y) {
        return this.scheduler.timeAxisViewModel.getDistanceForDuration(y - x);
    }

}