import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';
import { SERVER_BCS_URL } from 'src/app/app.constants';
import { createRequestOption } from 'src/app/shared/util/request-util';
import { IRoadMapBcs } from '../../model/road-map-bcs.model';
import { TaskableEntityTypeService } from '../../shared/services/taskable-entity-type.service';
import { DeployedVersionedSystemService } from 'src/app/bmcs/entity/deployed-versioned-system/deployed-versioned-system.service';
import { VersionedSystemService } from 'src/app/bmcs/entity/versioned-system/versioned-system.service';
import { SystemService } from 'src/app/bmcs/entity/system/system.service';
import { DeployedCenterService } from 'src/app/bmcs/entity/deployed-center/deployed-center.service';
import { DomaineService } from 'src/app/entities/domaine/domaine.service';
import { DisplayTaskService } from '../display-task/display-task.service';
import { ITask } from '../../model/task/Itask.model';
import { Baseline } from '../../model/enumeration/baseline.enum';
import { BcsNgHistoryService } from '../history/bcs-ng-history.service';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { Utils } from '../../utils/util';
import { IDomaine } from 'src/app/shared/model/entity/domaine.model';
import { DisplayRule } from '../../model/display-rule.model';

type EntityResponseType = HttpResponse<IRoadMapBcs>;
type EntityArrayResponseType = HttpResponse<IRoadMapBcs[]>;

@Injectable({ providedIn: 'root' })
export class RoadMapBcsService {
  public resourceUrl = SERVER_BCS_URL + 'api/road-maps';

  constructor(
    protected http: HttpClient,
    protected bmcsBaseService: BmcsBaseService,
    protected deployedVersionedSystemService: DeployedVersionedSystemService,
    protected versionedSystemService: VersionedSystemService,
    protected systemService: SystemService,
    protected deployedCenterService: DeployedCenterService,
    protected domaineService: DomaineService,
    protected displayTaskService: DisplayTaskService,
    protected historicService: BcsNgHistoryService,
  ) { }

  create(roadMap: IRoadMapBcs): Observable<EntityResponseType> {
    return this.http.post<IRoadMapBcs>(this.resourceUrl, roadMap, { observe: 'response' });
  }

  update(roadMap: IRoadMapBcs): Observable<EntityResponseType> {
    return this.http.put<IRoadMapBcs>(this.resourceUrl, roadMap, { observe: 'response' });
  }

  find(id: string): Observable<any> {
    return this.http.get<IRoadMapBcs>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  getTasksRpsObjects(id: string, date?: Date): Observable<HttpResponse<{ [key: string]: string[] }>> {
    let url = `${this.resourceUrl}/tasks-rps-objects/${id}`;
    if (date) {
      url += `?date=${date.toString()}`
    }
    return this.http.get<{ [key: string]: string[] }>(url, { observe: 'response' });
  }

  query(req?: any, baseline?: string): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    let url = this.resourceUrl;
    if (baseline) url += `?baseline=${baseline}`;
    return this.http.get<IRoadMapBcs[]>(url, { params: options, observe: 'response' });
  }

  delete(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }
  erase(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/erase/${id}`, { observe: 'response' });
  }

  /** Complete roadmap display tasks */
  async completeDisplayTasks(roadMap: IRoadMapBcs, baseline?: Baseline): Promise<IRoadMapBcs> {
    const roadMapFromHistory = await this.roadMapByBaseline(roadMap, baseline);
    if (!roadMapFromHistory) return null;
    let maxRpsMoment = undefined;
    if (baseline && baseline !== Baseline.NONE)
      maxRpsMoment = roadMapFromHistory.lastModifiedDate ?? roadMapFromHistory.createdDate;
    const maxRpsDate = maxRpsMoment ? new Date(maxRpsMoment.toString()) : undefined;
    const roadMapWithRpsObjects = await this.addDisplayTaskRpsObjects(roadMapFromHistory, maxRpsDate);
    const roadMapWithObjectsfromBmcs = await this.addDisplayTaskObjectsFromBMCS(roadMapWithRpsObjects);
    return roadMapWithObjectsfromBmcs;
  }

  private async roadMapByBaseline(roadMap: IRoadMapBcs, baseline: Baseline): Promise<IRoadMapBcs> {
    if (baseline === Baseline.NONE || !baseline) return roadMap;
    return new Promise(resolve => {
      this.historicService.findLatestByEntity(roadMap.id, baseline, true).subscribe(res => {
        const parsedHistoricObject = res.body ? JSON.parse(res.body.object) as IRoadMapBcs : null;
        resolve(parsedHistoricObject);
      });
    });
  }

  /** Fetch up to the last two previous rps updates of the underlying object */
  private async addDisplayTaskRpsObjects(roadMap: IRoadMapBcs, maxDate: Date): Promise<IRoadMapBcs> {
    return new Promise(resolve => {
      this.getTasksRpsObjects(roadMap.id, maxDate).subscribe(res => {
        const rpsObjectsMap = res.body;
        roadMap.tasks?.forEach(t => {
          const parsedRpsObjects = (rpsObjectsMap[t.id] ?? []).map(s => JSON.parse(s) as ITask);
          t.rpsObjects = parsedRpsObjects;
        });
        resolve(roadMap);
      });
    });
  }

  /** BMCS objects are stored in a different database from display tasks, so they must be retrieved for corresponding tasks */
  private async addDisplayTaskObjectsFromBMCS(roadMap: IRoadMapBcs): Promise<IRoadMapBcs> {
    if (roadMap) {
      const systems = this.bmcsBaseService.getSystems;
      const sv = this.bmcsBaseService.getVersionedSystems;
      const cd = this.bmcsBaseService.getDeployedCenters;
      const domaines = this.bmcsBaseService.getDomaines();

      for (const t of roadMap?.tasks) {
        if (TaskableEntityTypeService.isDomaine(t.type?.id)) {
          t.object = domaines.find(s => s.id === t.object?.id);
        }
        else if (TaskableEntityTypeService.isVersionedSystem(t.type?.id)) {
          t.object = sv.find(sv => sv.id === t.object?.id);
        }
        else if (TaskableEntityTypeService.isSystem(t.type?.id)) {
          t.object = systems.find(s => s.id === t.object?.id);
        }
        else if (TaskableEntityTypeService.isDeployedCenter(t.type?.id)) {
          t.object = cd.find(cd => cd.id === t.object?.id);
        }
      }
      return roadMap;
    }
    return null;
  }

  cloneRoadMap(roadMapToClone: IRoadMapBcs, domains: IDomaine[], title: string): IRoadMapBcs {
    const cloneUuid = Utils.createUUID();
    const cloneDisplayConfUuid = Utils.createUUID();

    const ruleClones: DisplayRule[] = [];
    roadMapToClone?.displayConfig?.displayRules.forEach(rule => {
      const ruleCloneId = Utils.createUUID();
      const ruleClone = {
        ...rule,
        id: ruleCloneId,
      };
      delete ruleClone.ownerId;
      ruleClones.push(ruleClone);
    });

    const roadMapClone = {
      ...roadMapToClone,
      displayConfig: {
        ...roadMapToClone.displayConfig,
        id: cloneDisplayConfUuid,
        displayRules: ruleClones,
      },
      id: cloneUuid,
      domainesId: JSON.stringify(domains.map(d => d.id)),
      name: title,
    }
    delete roadMapClone.createdBy;
    delete roadMapClone.createdByUserName;
    delete roadMapClone.createdDate;
    delete roadMapClone.deleted;
    delete roadMapClone.deleteBy;
    delete roadMapClone.deleteDate;
    delete roadMapClone.lastModifiedBy;
    delete roadMapClone.lastModifiedByUserName;
    delete roadMapClone.lastModifiedDate;
    delete roadMapClone.modifyBlockingDate;
    delete roadMapClone.modifyBlockingBy;
    return roadMapClone;
  }
}
