import { Component, OnInit } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { Model, Scheduler } from '@bryntum/scheduler';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { DisplayConfigBryntum } from '../../bryntum/display-config';
import { SchedulerStoresHandler } from '../../bryntum/scheduler-stores';
import { MultiColorsEventConfig } from '../../../../model/bryntum/display-configuration/multi-colors-event-config';
import { DisplayConfTaskProperties } from 'src/app/bcs_ng/model/bryntum/display-configuration/display-conf-task-properties';

@Component({
  templateUrl: './display-configuration-dialog.component.html',
  styleUrls: ['../../bryntum/bryntum.scss', './display-configuration-dialog.scss', '../display-configuration-rules/display-configuration-rules-dialog.scss'],
  providers: [SchedulerStoresHandler]
})
export class DisplayConfigurationDialogcomponent implements OnInit {

  scheduler: Scheduler;
  eventRecord: Model;
  childEvents: Model[];
  settingForm = this.fb.group({
    bgColor: [null],
    shape: [null],
    position: ['undefined', [Validators.required]]
  });
  isMilestone: boolean = false;
  isIndepMilestone: boolean = false;

  childrenEvents: Model[] = [];
  multiColorsSelected: MultiColorsEventConfig[] = [];

  fontSize: number = undefined;
  defaultFontSize: number = 16;

  constructor(
    public activeModal: NgbActiveModal,
    private fb: FormBuilder,
    protected schedulerStoresHandler: SchedulerStoresHandler
  ) { }

  // ----------------------------------- Component lifecycle -----------------------------------
  ngOnInit(): void {
    if (TaskableEntityTypeService.isMilestone(this.eventRecord.getData('type')?.id)) this.isMilestone = true;
    const isIndepMilestone: boolean = this.eventRecord.getData('independantMilestone');
    this.isIndepMilestone = isIndepMilestone;
    let properties = DisplayConfigBryntum.getCurrentDisplayConfTaskProperties(this.eventRecord);
    if (properties?.multiColorsEventConfigs) this.multiColorsSelected = properties?.multiColorsEventConfigs
    properties ? this.setConfig(properties) : this.setDefaultConfig();
    if (!this.isMilestone) this.getChildEvent();
  }

  setDefaultConfig() {
    this.setDefaultConfigColor();
    this.setDefaultConfigShape();
  }
  setDefaultConfigColor() {
    this.settingForm.controls['bgColor'].setValue(null);
  }
  setDefaultConfigShape() {
    this.settingForm.controls['shape'].setValue(null);
  }
  setDefaultConfigPosition() {
    this.settingForm.controls['position'].setValue('undefined');
  }
  setConfig(properties: DisplayConfTaskProperties) {
    let color = properties?.bgColor;
    this.settingForm.controls['bgColor'].setValue(color ?? null);
    let shape = properties.shape ? properties.shape.replace('b-fa fa-solid fa-', '') : null;
    this.settingForm.controls['shape'].setValue(shape);
    this.settingForm.controls['position'].setValue(properties?.position ?? 'undefined');
  }

  cancel(): void {
    this.activeModal.close(null);
  }

  save(): void {
    let properties = this.getOverrideDisplayConfig();
    this.setStyle(this.eventRecord, properties);
    if (this.eventRecord.getData('cloneId')) this.applyToClone(properties);
    properties = {
      ...properties,
      eventFontSize: this.fontSize,
    };
    this.activeModal.close({ idEvent: this.eventRecord.getData('id'), properties: properties });
  }

  public getOverrideDisplayConfig(): DisplayConfTaskProperties {
    return {
      resourceFontColor: null,
      resourceFontSize: null,
      eventFontSize: null,
      rowHeight: null,
      bgColor: this.settingForm.controls['bgColor']?.value,
      shape: this.settingForm.controls['shape']?.value ? 'b-fa fa-solid fa-' + this.settingForm.controls['shape']?.value : null,
      position: this.settingForm.controls['position']?.value != 'undefined' ? this.settingForm.controls['position']?.value : null,
      multiColorsEventConfigs: this.multiColorsSelected[0]?.startOffsetEventId ? this.multiColorsSelected : null,
      delayArrowConf: null,
      delayRpsArrowConf: null
    }
  }

  setStyle(event: Model, properties: DisplayConfTaskProperties) {
    event.set('eventColor', properties?.bgColor);
    event.set('iconCls', properties?.shape);
    event.set('label', properties?.position);
    event.set('offsetMultiColorEventConfig', properties?.multiColorsEventConfigs);
    event.set('fontSize', properties?.eventFontSize);
  }

  public applyToClone(properties: DisplayConfTaskProperties) {
    let clone = this.scheduler.eventStore.getById(this.eventRecord.getData('cloneId'));
    this.setStyle(clone, properties);
  }


  getChildEvent() {
    this.childrenEvents = SchedulerStoresHandler.getChildEventByEvent(this.scheduler, this.eventRecord);
    if (this.multiColorsSelected?.length == 0) this.addEmptyMultiColorRule();
  }

  addEmptyMultiColorRule() {
    this.multiColorsSelected.push({ ...new MultiColorsEventConfig() });
  }

  removeMultiColorRule(index: number) {
    this.multiColorsSelected.splice(index, 1);
  }

  isChildDisabled(index: number, optionDate: Date) {
    let startOffsetId = this.multiColorsSelected[index].startOffsetEventId;
    if (!startOffsetId) return false;
    return this.childrenEvents.find(c => c.id === startOffsetId).getData('startDate') >= optionDate;
  }

  increaseFontSize(): void {
    if (this.fontSize) this.fontSize += 1;
    else this.fontSize = this.defaultFontSize + 1;
  }

  reduceFontSize(): void {
    if (this.fontSize && this.fontSize > 5) this.fontSize -= 1;
    else this.fontSize = this.defaultFontSize - 1;
  }

  setFontSizeToDefault(): void {
    this.fontSize = undefined;
  }
}