import { Component, Inject, OnInit } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { IBryntumEvent } from 'src/app/bcs_ng/model/bryntum/bryntum-event.model';
import { IBryntumOverridableData } from 'src/app/bcs_ng/model/bryntum/bryntum-overridable-data.model';
import { BryntumRecordState, IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { DisplayTaskDateValidators } from 'src/app/bcs_ng/shared/validators/display-task-date-validators';
import { DateAdapter, MAT_DATE_FORMATS, MAT_DATE_LOCALE } from '@angular/material/core';
import {
  MAT_MOMENT_DATE_FORMATS,
  MomentDateAdapter,
  MAT_MOMENT_DATE_ADAPTER_OPTIONS,
} from '@angular/material-moment-adapter';

@Component({
  templateUrl: './display-task-update-dialog.component.html',
  styleUrls: ['../../bryntum/bryntum.scss'],
  providers: [
    {
      provide: DateAdapter,
      useClass: MomentDateAdapter,
      deps: [MAT_DATE_LOCALE, MAT_MOMENT_DATE_ADAPTER_OPTIONS],
    },
    { provide: MAT_DATE_FORMATS, useValue: MAT_MOMENT_DATE_FORMATS },
  ]
})
export class DisplayTaskUpdateDialogcomponent implements OnInit {
  record: IBryntumRecordState;
  eventRecords: IBryntumRecordState[];
  successors: IBryntumEvent[];
  // events: IBryntumEvent[];
  defaultValues: IBryntumOverridableData;

  startAt: Date = new Date();

  editForm = this.fb.group({
    id: [],
    name: [],
    shortName: [null, [Validators.maxLength(25)]],
    startDate: [],
    endDate: [],
    predecessors: [],
  });

  updateInfo = 'Les champs du formulaire appartiennent à la tâche dans le contexte de la feuille de route. ' +
    'L\'entité sur laquelle la tâche est basée n\'est pas affectée.\n' +
    'Si les valeurs sont vides, ce sont les valeurs de la base calendaire qui sont utilisées pour l\'affichage.\n\n' +
    'Le cas du champ "Prédécesseurs" est particulier, consulter l\'aide de celui-ci pour plus d\'informations';

  startDateInfo = 'Attention, modifier cette date affectera potentiellement la date d\'autres tâches de type jalon sur la même feuille de route :\n' +
    '  - Les tâches basées sur le même jalon seront mises à jour avec la même date\n' +
    '  - Les successeurs seront potentiellement mis-à-jour, en fonction des dates de leurs autres prédécesseurs \n\n' +
    'Il s\'agit d\'un proecessus récursif.';

  constructor(
    public activeModal: NgbActiveModal,
    private fb: FormBuilder,
    private _adapter: DateAdapter<any>,
    @Inject(MAT_DATE_LOCALE) private _locale: string
  ) { }

  // ----------------------------------- Component lifecycle -----------------------------------
  ngOnInit(): void {
    // this.initEvents();
    this.updateDefaultValues();
    this.updateForm();
    this.setDatePickerInitialDate();
    this._locale = 'fr';
    this._adapter.setLocale(this._locale);
  }

  // ----------------------------------- Modal actions -----------------------------------
  cancel(): void {
    this.activeModal.dismiss();
  }

  save(): void {
    const updatedRecord = this.recordFromForm();
    const newDateFormValue = this.editForm.get('startDate').value;
    const newDate = newDateFormValue ? new Date(newDateFormValue) : undefined;
    const milestoneDateModified = this.isMilestone() && this.record.event.startDate?.getTime() !== newDate?.getTime();
    const overridingDateRemoved = newDate === undefined;
    this.activeModal.close({
      newRecordValue: updatedRecord,
      milestoneDateModified,
      overridingDateRemoved,
    });
  }

  focus(): void {
    const element = document.querySelectorAll('[aria-label="multiselect-search"]') as NodeListOf<HTMLInputElement>;
    if (element[0]) element[0].focus();
    this.editForm.get('predecessors').markAsTouched();
  }

  emptyForm(): void {
    this.editForm.patchValue({
      name: null,
      shortName: null,
      startDate: null,
      endDate: null,
    });
  }

  updateForm(): void {
    this.editForm.patchValue({
      id: this.record.id,
      name: this.record.resource.name,
      shortName: this.record.event.name,
      startDate: this.record.event.startDate,
      endDate: this.record.event.endDate,
    });
    if (TaskableEntityTypeService.isGenericTask(this.record.resource.typeId)) {
      this.editForm.setValidators(DisplayTaskDateValidators.dateLessThan(
        'startDate',
        'endDate',
        new Date(this.record.resource.object.startDate?.toString()),
        new Date(this.record.resource.object.endDate?.toString()),
        { startLessThanEnd: true },
      ));
    }
  }

  /** Determines whether the task is based on a milestone, and if its date was modified with an overriding value */
  milestoneDateModified(): boolean {
    if (!this.isMilestone()) return false;
    const newDateFormValue = this.editForm.get('startDate').value;
    const newDate = newDateFormValue ? new Date(newDateFormValue) : undefined;
    return this.record.event.startDate?.getTime() !== newDate?.getTime() && newDate !== undefined;
  }

  updateDefaultValues(): void {
    this.defaultValues = {
      name: this.record.resource.object.name,
      shortName: this.record.resource.object.shortName,
      startDate: new Date(this.record.resource.object.startDate?.toString()),
      endDate: new Date(this.record.resource.object.endDate?.toString()),
    }
  }

  private setDatePickerInitialDate(): void {
    this.startAt = this.editForm.get('startDate').value ?
      new Date(this.editForm.get('startDate').value) :
      new Date(this.defaultValues.startDate);
  }

  // ----------------------------------- Utils -----------------------------------

  recordFromForm(): BryntumRecordState {
    const name = this.editForm.get(['name'])!.value
    const shortName = this.editForm.get(['shortName'])!.value;
    let endDate, startDate;
    if (this.isMilestone()) endDate = this.editForm.get(['startDate'])!.value;
    if (this.isGenericTask()) endDate = this.editForm.get(['endDate'])!.value;
    startDate = this.editForm.get(['startDate'])!.value;

    const overridden = name || shortName || startDate || endDate;

    const resultData = {
      ...this.record,
      resource: {
        ...this.record.resource,
        name,
        overridden,
      },
      event: {
        ...this.record.event,
        name: shortName,
        startDate: startDate ? new Date(startDate) : undefined,
        endDate: endDate ? new Date(endDate) : undefined,
      }
    }
    return new BryntumRecordState(
      resultData.id,
      false,
      true,
      resultData.resource,
      resultData.event
    )
  }

  startDateLabel(): string {
    return TaskableEntityTypeService.isMilestone(this.record.resource.typeId) ? 'Date' : 'Date de début';
  }

  oneDateDirtyOrTouched(): boolean {
    return this.editForm?.get('startDate')?.dirty ||
      this.editForm?.get('startDate')?.touched ||
      this.editForm?.get('endDate')?.dirty ||
      this.editForm?.get('endDate')?.touched;
  }

  isGenericTask(): boolean {
    return TaskableEntityTypeService.isGenericTask(this.record.resource.typeId);
  }

  isMilestone(): boolean {
    return TaskableEntityTypeService.isMilestone(this.record.resource.typeId);
  }

  isOperation(): boolean {
    return TaskableEntityTypeService.isOperation(this.record.resource.typeId);
  }

  isTaskFromBmcs(): boolean {
    return TaskableEntityTypeService.isTaskFromBmcs(this.record.resource.typeId);
  }
}