import { AfterContentChecked, ChangeDetectorRef, Component, Inject, OnInit } from "@angular/core";
import { FormBuilder, Validators } from '@angular/forms';
import { MAT_MOMENT_DATE_ADAPTER_OPTIONS, MAT_MOMENT_DATE_FORMATS, MomentDateAdapter } from '@angular/material-moment-adapter';
import { DateAdapter, MAT_DATE_FORMATS, MAT_DATE_LOCALE } from '@angular/material/core';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { IBryntumEvent } from 'src/app/bcs_ng/model/bryntum/bryntum-event.model';
import { IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';
import { IBryntumResource } from 'src/app/bcs_ng/model/bryntum/bryntum-resource.model';
import { Utils } from 'src/app/bcs_ng/utils/util';

@Component({
  selector: 'update-independant-task',
  templateUrl: './update-independant-task.component.html',
  providers: [
    {
      provide: DateAdapter,
      useClass: MomentDateAdapter,
      deps: [MAT_DATE_LOCALE, MAT_MOMENT_DATE_ADAPTER_OPTIONS],
    },
    { provide: MAT_DATE_FORMATS, useValue: MAT_MOMENT_DATE_FORMATS },
  ]
})
export class UpdateIndependantTaskComponent implements OnInit, AfterContentChecked {
  record: IBryntumRecordState;
  editForm = this.fb.group({
    id: [],
    independantMilestone: [false],
    name: [null, [Validators.required]],
    shortName: [null, [Validators.required, Validators.maxLength(25)]],
    date: [null],
  });

  overallText = 'La tâche créée ne sera basée sur aucun objet de la BCS.';
  checkboxInfo = 'Si la tâche est un jalon indépendant, alors elle aura une date fixée. Sinon, ses dates de début et de fin seront déterminées par ses enfants.';

  constructor(
    public activeModal: NgbActiveModal,
    private fb: FormBuilder,
    private cdRef: ChangeDetectorRef,
    private _adapter: DateAdapter<any>,
    @Inject(MAT_DATE_LOCALE) private _locale: string
  ) { }

  ngOnInit(): void {
    this.cdRef.detach();
    if (this.record) this.updateForm();
    this._locale = 'fr';
    this._adapter.setLocale(this._locale);
  }

  ngAfterContentChecked(): void {
    this.cdRef.detectChanges();
  }

  dateValid(): boolean {
    const independantMilestone: boolean = this.editForm.get('independantMilestone').value;
    if (independantMilestone) {
      const dateValue = this.editForm.get('date').value;
      return dateValue !== null && dateValue !== undefined;
    } else {
      return true;
    }
  }

  displayDatesError(): boolean {
    const independantMilestone: boolean = this.editForm.get('independantMilestone').value;
    return independantMilestone &&
      (this.editForm.get('date').touched || this.editForm.get('date').dirty) &&
      !this.editForm.get('date').value;
  }

  cancel(): void {
    this.activeModal.dismiss();
  }

  save(): void {
    const newRecord = this.buildRecord();
    this.activeModal.close(newRecord);
  }

  private updateForm(): void {
    this.editForm.patchValue({
      name: this.record.resource.name,
      independantMilestone: this.record.resource.independantMilestone,
      shortName: this.record.event.name,
      date: this.record.event.startDate,
    });
  }

  private buildRecord(): IBryntumRecordState {
    const id = this.record?.id ?? Utils.createUUID();
    const resource: IBryntumResource = {
      id,
      name: this.editForm.get('name').value,
      hidden: false,
      deleted: false,
      displayed: true,
      typeId: '',
      parentId: this.record?.resource.parentId ?? null,
      parentIndex: this.record?.resource.parentIndex ?? undefined,
      readOnly: false,
      createdBy: this.record?.resource.createdBy ?? null,
      object: null,
      rpsObjects: [],
      expanded: this.record?.resource.expanded ?? false,
      cls: this.record?.resource.cls ?? 'independant-task',
      independantMilestone: this.record ? this.record.resource.independantMilestone : this.editForm.get('independantMilestone').value,
      fontSize: this.record?.resource?.fontSize,
      fontColor: this.record?.resource?.fontColor,
      rowHeight: this.record?.resource?.rowHeight,
    }
    const event: IBryntumEvent = {
      id,
      name: this.editForm.get('shortName').value,
      locationId: this.record?.event.locationId ?? null,
      startDate: this.editForm.get('date').value ? new Date(this.editForm.get('date').value) : null,
      endDate: this.editForm.get('date').value ? new Date(this.editForm.get('date').value) : null,
      readOnly: true,
      parentId: this.record?.event.parentId ?? null,
      object: null,
      type: null,
      hidePredecessors: false,
      cls: this.record?.event.cls ?? '',
      delayEvent: false,
      independantMilestone: this.record ? this.record.resource.independantMilestone : this.editForm.get('independantMilestone').value,
    }
    return {
      id,
      toErase: false,
      isUpdated: true,
      resource,
      event
    }
  }
}