import { Component, Input, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';

import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';
import { GenericTask, IGenericTask } from 'src/app/bcs_ng/model/task/generic-task.model';
import { GenericTaskService } from './generic-task.service';
import { AccountService } from 'src/app/core/auth/account.service';
import * as moment from 'moment';
import { GenericTaskDateValidators } from 'src/app/bcs_ng/shared/validators/generic-task-date-validators';

@Component({
  selector: 'jhi-generic-task-update',
  templateUrl: './generic-task-update.component.html'
})
export class GenericTaskUpdateComponent implements OnInit {
  @Input()
  genericTask: IGenericTask;
  isSaving = false;

  editForm = this.fb.group({
    id: [],
    name: [],
    shortName: [null, [Validators.maxLength(25)]],
    startDate: [null, Validators.required],
    endDate: [null, Validators.required],
    comment: [null, Validators.maxLength(255)],
  }, {
    validators: [GenericTaskDateValidators.dateLessThan('startDate', 'endDate', { startLessThanEnd: true })]
  });

  canUpdate: boolean = false;
  tryUpdate: boolean = false;

  constructor(
    protected genericTaskService: GenericTaskService,
    protected activatedRoute: ActivatedRoute,
    protected accountService: AccountService,
    protected activeModal: NgbActiveModal,
    private fb: FormBuilder,
    protected eventManager: JhiEventManager,
  ) { }

  ngOnInit(): void {
    if (this.genericTask != null) this.updateForm(this.genericTask);
  }

  updateForm(genericTask: IGenericTask): void {
    this.editForm.patchValue({
      id: genericTask.id,
      name: genericTask.name,
      shortName: genericTask.shortName,
      startDate: genericTask.startDate ? new Date(genericTask.startDate?.toString()) : undefined,
      endDate: genericTask.endDate ? new Date(genericTask.endDate?.toString()) : undefined,
      comment: genericTask.comment,
    });
  }

  cancel(): void {
    if (this.accountService.userIdentity?.login === this.genericTask?.modifyBlockingBy)
      this.genericTaskService.freeUpdate(this.genericTask).subscribe(() => { });
    this.activeModal.dismiss();
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const genericTask = this.createFromForm();
    if (genericTask.id !== undefined && genericTask.id !== null) {
      this.subscribeToSaveResponse(this.genericTaskService.update(genericTask));
    } else {
      genericTask.id = undefined;
      this.subscribeToSaveResponse(this.genericTaskService.create(genericTask));
    }
  }

  private createFromForm(): IGenericTask {
    return {
      ...new GenericTask(),
      id: this.editForm.get(['id'])!.value,
      name: this.editForm.get(['name'])!.value,
      shortName: this.editForm.get(['shortName'])!.value,
      startDate: moment(this.editForm.get(['startDate'])!.value),
      endDate: moment(this.editForm.get(['endDate'])!.value),
      comment: this.editForm.get(['comment'])!.value,
    };
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IGenericTask>>): void {
    result.subscribe(
      (res) => this.onSaveSuccess(res.body),
      () => this.onSaveError()
    );
  }

  protected onSaveSuccess(res: IGenericTask): void {
    this.isSaving = false;
    this.eventManager.broadcast({ name: 'genericTaskListModification', content: res.id });
    this.activeModal.close();
  }

  protected onSaveError(): void {
    this.isSaving = false;
  }

  protected freeUpdate(doesUpdate: boolean) {
    if (this.accountService.userIdentity?.login === this.genericTask?.modifyBlockingBy)
      this.genericTaskService.freeUpdate(this.genericTask).subscribe((res) => {
        this.genericTask = res.body;
        this.importActionForUpdate(doesUpdate);
      });
  }

  protected importActionForUpdate(doesUpdate: boolean): void {
    if (doesUpdate) this.updateForm(this.genericTask);
    this.canUpdate = true;
    this.editForm.enable()
    this.tryUpdate = false;
  }

  oneDateDirtyOrTouched(): boolean {
    return this.editForm?.get('startDate')?.dirty ||
      this.editForm?.get('startDate')?.touched ||
      this.editForm?.get('endDate')?.dirty ||
      this.editForm?.get('endDate')?.touched;
  }
}
