import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { SERVER_BCS_URL } from 'src/app/app.constants';
import { createRequestOption } from 'src/app/shared/util/request-util';
import { map } from 'rxjs/operators';
import { IGenericTask } from 'src/app/bcs_ng/model/task/generic-task.model';
import { DATE_FORMAT } from 'src/app/shared/constants/input.constants';
import * as moment from 'moment';

type EntityResponseType = HttpResponse<IGenericTask>;
type EntityArrayResponseType = HttpResponse<IGenericTask[]>;

@Injectable({ providedIn: 'root' })
export class GenericTaskService {
  public resourceUrl = SERVER_BCS_URL + 'api/generic-tasks';

  constructor(protected http: HttpClient) { }

  create(genericTask: IGenericTask): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(genericTask);
    return this.http
      .post<IGenericTask>(this.resourceUrl, copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  blockUpdate(genericTask: IGenericTask): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(genericTask);
    return this.http
      .put<IGenericTask>(this.resourceUrl + '/block/update', copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  freeUpdate(genericTask: IGenericTask): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(genericTask);
    return this.http
      .put<IGenericTask>(this.resourceUrl + '/free/update', copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  update(genericTask: IGenericTask): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(genericTask);
    return this.http
      .put<IGenericTask>(this.resourceUrl, copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  find(id: string): Observable<EntityResponseType> {
    return this.http
      .get<IGenericTask>(`${this.resourceUrl}/${id}`, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  query(getAll: boolean, req?: any): Observable<EntityArrayResponseType> {
    const url = getAll ? this.resourceUrl + '/get/all' : this.resourceUrl;
    const options = createRequestOption(req);
    return this.http
      .get<IGenericTask[]>(url, { params: options, observe: 'response' })
      .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  erase(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/erase/${id}`, { observe: 'response' });
  }

  protected convertDateFromClient(genericTask: IGenericTask): IGenericTask {
    const copy: IGenericTask = Object.assign({}, genericTask, {
      modifyBlockingDate: genericTask.modifyBlockingDate && genericTask.modifyBlockingDate.isValid() ? genericTask.modifyBlockingDate.format(DATE_FORMAT) : undefined,
      startDate: genericTask.startDate && genericTask.startDate.isValid() ? genericTask.startDate.format(DATE_FORMAT) : undefined,
      endDate: genericTask.endDate && genericTask.endDate.isValid() ? genericTask.endDate.format(DATE_FORMAT) : undefined,
    });
    return copy;
  }

  protected convertDateFromServer(res: EntityResponseType): EntityResponseType {
    if (res.body) {
      res.body.modifyBlockingDate = res.body.modifyBlockingDate ? moment(res.body.modifyBlockingDate) : undefined;
      res.body.startDate = res.body.startDate ? moment(res.body.startDate) : undefined;
      res.body.endDate = res.body.endDate ? moment(res.body.endDate) : undefined;
    }
    return res;
  }

  protected convertDateArrayFromServer(res: EntityArrayResponseType): EntityArrayResponseType {
    if (res.body) {
      res.body.forEach((genericTask: IGenericTask) => {
        genericTask.modifyBlockingDate = genericTask.modifyBlockingDate ? moment(genericTask.modifyBlockingDate) : undefined;
        genericTask.startDate = genericTask.startDate ? moment(genericTask.startDate) : undefined;
        genericTask.endDate = genericTask.endDate ? moment(genericTask.endDate) : undefined;
      });
    }
    return res;
  }
}
