import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import * as moment from 'moment';
import { SERVER_BCS_URL } from 'src/app/app.constants';
import { DATE_FORMAT } from 'src/app/shared/constants/input.constants';
import { IMilestone } from 'src/app/bcs_ng/model/milestone.model';
import { createRequestOption } from 'src/app/shared/util/request-util';
import { Baseline } from 'src/app/bcs_ng/model/enumeration/baseline.enum';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { BcsNgHistoryService } from '../../history/bcs-ng-history.service';


type EntityResponseType = HttpResponse<IMilestone>;
type EntityArrayResponseType = HttpResponse<IMilestone[]>;

@Injectable({ providedIn: 'root' })
export class MilestoneService {
  public resourceUrl = SERVER_BCS_URL + 'api/milestones';

  constructor(protected http: HttpClient,
    protected bcsNgHistoryService: BcsNgHistoryService) { }

  create(milestone: IMilestone): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(milestone);
    return this.http
      .post<IMilestone>(this.resourceUrl, copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  blockUpdate(milestone: IMilestone): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(milestone);
    return this.http
      .put<IMilestone>(this.resourceUrl + '/block/update', copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  freeUpdate(milestone: IMilestone): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(milestone);
    return this.http
      .put<IMilestone>(this.resourceUrl + '/free/update', copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  update(milestone: IMilestone): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(milestone);
    return this.http
      .put<IMilestone>(this.resourceUrl, copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  find(id: string): Observable<EntityResponseType> {
    return this.http
      .get<IMilestone>(`${this.resourceUrl}/${id}`, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  query(getAll: boolean = false, req?: any, idList?: string[]): Observable<EntityArrayResponseType> {
    let url = getAll ? this.resourceUrl + '/get/all' : this.resourceUrl;
    if (idList?.length) url += `?ids=${idList.join(';')}`;
    const options = createRequestOption(req);
    return this.http
      .get<IMilestone[]>(url, { params: options, observe: 'response' })
      .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
  }

  getAllNumberIds(): Observable<number[]> {
    return this.http.get<number[]>(this.resourceUrl + '/get/allNumberIds');
  }

  queryByBaseline(baseline: Baseline) {
    return this.bcsNgHistoryService.queryByBaseline(TaskableEntityTypeService.getIdMilestone(), baseline);
  }

  migrate(milestones: IMilestone[]): Observable<EntityArrayResponseType> {
    return this.http
      .post<IMilestone[]>(this.resourceUrl + '/migrate', milestones, { observe: 'response' })
      .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
  }

  delete(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  erase(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/erase/${id}`, { observe: 'response' });
  }

  protected convertDateFromClient(milestone: IMilestone): IMilestone {
    const copy: IMilestone = Object.assign({}, milestone, {
      modifyBlockingDate: milestone.modifyBlockingDate && milestone.modifyBlockingDate.isValid() ? milestone.modifyBlockingDate.format(DATE_FORMAT) : undefined,
      startDate: milestone.startDate && milestone.startDate.isValid() ? milestone.startDate.format(DATE_FORMAT) : undefined,
      endDate: milestone.endDate && milestone.endDate.isValid() ? milestone.endDate.format(DATE_FORMAT) : undefined,
      contractDate: milestone.contractDate ? milestone.contractDate.format(DATE_FORMAT) : null,
    });
    return copy;
  }

  protected convertDateFromServer(res: EntityResponseType): EntityResponseType {
    if (res.body) {
      res.body.modifyBlockingDate = res.body.modifyBlockingDate ? moment(res.body.modifyBlockingDate) : undefined;
      res.body.startDate = res.body.startDate ? moment(res.body.startDate) : undefined;
      res.body.endDate = res.body.endDate ? moment(res.body.endDate) : undefined;
      res.body.contractDate = res.body.contractDate ? moment(res.body.contractDate) : undefined;
    }
    return res;
  }

  protected convertDateArrayFromServer(res: EntityArrayResponseType): EntityArrayResponseType {
    if (res.body) {
      res.body.forEach((milestone: IMilestone) => {
        milestone.modifyBlockingDate = milestone.modifyBlockingDate ? moment(milestone.modifyBlockingDate) : undefined;
        milestone.startDate = milestone.startDate ? moment(milestone.startDate) : undefined;
        milestone.endDate = milestone.endDate ? moment(milestone.endDate) : undefined;
        milestone.contractDate = milestone.contractDate ? moment(milestone.contractDate) : null;
      });
    }
    return res;
  }
}
