import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { SERVER_BCS_URL } from 'src/app/app.constants';
import { createRequestOption } from 'src/app/shared/util/request-util';
import { IOperation } from 'src/app/bcs_ng/model/operation.model';
import { map } from 'rxjs/operators';
import { DATE_FORMAT } from 'src/app/shared/constants/input.constants';
import * as moment from 'moment';
import { BcsNgHistoryService } from '../../history/bcs-ng-history.service';
import { Baseline } from 'src/app/bcs_ng/model/enumeration/baseline.enum';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';

type EntityResponseType = HttpResponse<IOperation>;
type EntityArrayResponseType = HttpResponse<IOperation[]>;

@Injectable({ providedIn: 'root' })
export class OperationService {
  public resourceUrl = SERVER_BCS_URL + 'api/operations';

  constructor(protected http: HttpClient, protected bcsNgHistoryService: BcsNgHistoryService) { }

  create(operation: IOperation): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(operation);
    return this.http
      .post<IOperation>(this.resourceUrl, copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  blockUpdate(operation: IOperation): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(operation);
    return this.http
      .put<IOperation>(this.resourceUrl + '/block/update', copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  freeUpdate(operation: IOperation): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(operation);
    return this.http
      .put<IOperation>(this.resourceUrl + '/free/update', copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  update(operation: IOperation): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(operation);
    return this.http
      .put<IOperation>(this.resourceUrl, copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  find(id: string): Observable<EntityResponseType> {
    return this.http
      .get<IOperation>(`${this.resourceUrl}/${id}`, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  query(getAll: boolean = false, req?: any): Observable<EntityArrayResponseType> {
    const url = getAll ? this.resourceUrl + '/get/all' : this.resourceUrl;
    const options = createRequestOption(req);
    return this.http
      .get<IOperation[]>(url, { params: options, observe: 'response' })
      .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
  }

  queryByBaseline(baseline: Baseline) {
    return this.bcsNgHistoryService.queryOperationAndMilestoneByBaseline(baseline);
  }

  findByVersionedSystemId(id: string) {
    return this.http
      .get<IOperation[]>(this.resourceUrl + '/get/byVersionedSystem/' + id, { observe: 'response' })
      .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
  }

  getAvailableManualId(): Observable<string> {
    return this.http.get<string>(this.resourceUrl + '/manual_id/available');
  }

  migrate(operations: IOperation[]): Observable<EntityArrayResponseType> {
    return this.http
      .post<IOperation[]>(this.resourceUrl + '/migrate', operations, { observe: 'response' })
      .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  erase(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/erase/${id}`, { observe: 'response' });
  }

  protected convertDateFromClient(operation: IOperation): IOperation {
    const copy: IOperation = Object.assign({}, operation, {
      modifyBlockingDate: operation.modifyBlockingDate && operation.modifyBlockingDate.isValid() ? operation.modifyBlockingDate.format(DATE_FORMAT) : undefined,
      startDate: operation.startDate && operation.startDate.isValid() ? operation.startDate.format(DATE_FORMAT) : undefined,
      endDate: operation.endDate && operation.endDate.isValid() ? operation.endDate.format(DATE_FORMAT) : undefined,
    });
    return copy;
  }

  protected convertDateFromServer(res: EntityResponseType): EntityResponseType {
    if (res.body) {
      res.body.modifyBlockingDate = res.body.modifyBlockingDate ? moment(res.body.modifyBlockingDate) : undefined;
      res.body.startDate = res.body.startDate ? moment(res.body.startDate) : undefined;
      res.body.endDate = res.body.endDate ? moment(res.body.endDate) : undefined;
      res.body?.milestones?.forEach(mil => {
        mil.contractDate = mil.contractDate ? moment(mil.contractDate) : undefined;
        mil.startDate = mil.startDate ? moment(mil.startDate) : undefined;
        mil.endDate = mil.endDate ? moment(mil.endDate) : undefined;
      })
    }
    return res;
  }

  protected convertDateArrayFromServer(res: EntityArrayResponseType): EntityArrayResponseType {
    if (res.body) {
      res.body.forEach((operation: IOperation) => {
        operation.modifyBlockingDate = operation.modifyBlockingDate ? moment(operation.modifyBlockingDate) : undefined;
        operation.startDate = operation.startDate ? moment(operation.startDate) : undefined;
        operation.endDate = operation.endDate ? moment(operation.endDate) : undefined;
      });
    }
    return res;
  }
}
