import { HttpResponse } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';
import { Subscription } from 'rxjs';
import { StatistiqueService } from 'src/app/admin/statistique/statistique.service';
import { StorageService, StoreKey } from 'src/app/blocks/storage/storage-service';
import { SystemService } from 'src/app/bmcs/entity/system/system.service';
import { IDeployedCenter } from 'src/app/bmcs/model/deployed-center.model';
import { IVersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { BmcsBaseComponent } from 'src/app/bmcs/shared/bmcs-base-component';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { AccountService } from 'src/app/core/auth/account.service';
import { DomaineService } from 'src/app/entities/domaine/domaine.service';
import { CruddomaineService } from 'src/app/entities/group-users/cruddomaine.service';
import { ICruddomaine } from 'src/app/shared/model/entity/cruddomaine.model';
import { IDomaine } from 'src/app/shared/model/entity/domaine.model';
import { Baseline, RpsValidationScope } from '../../model/enumeration/baseline.enum';
import { IMilestone } from '../../model/milestone.model';
import { IOperation } from '../../model/operation.model';
import { ImportBcsDialogComponent } from '../../shared/import-bcs/import-dialog.component';
import { BcsNgHistoricScopeUpdateComponent } from '../history/bcs-ng-historic-scope-update.component';
import { MilestoneService } from './milestone/milestone.service';
import { OperationService } from './operation/operation.service';


@Component({
  selector: 'jhi-table-bcs',
  templateUrl: './table-bcs-ng.component.html',
  styleUrls: ['./table-bcs-ng.scss']
})
export class TableBcsComponent extends BmcsBaseComponent implements OnInit {

  operations?: IOperation[];
  milestones?: IMilestone[];
  crudDomaines?: [ICruddomaine[]];
  domains?: IDomaine[];
  overseenDomains?: IDomaine[];
  versionedSystems?: IVersionedSystem[] = [];
  deployedCenters?: IDeployedCenter[] = [];
  eventSubscriber?: Subscription;

  baseline: Baseline;
  Baseline = Baseline;

  load: boolean = false;

  constructor(protected operationService: OperationService,
    protected milestoneService: MilestoneService,
    protected systemService: SystemService,
    protected modalService: NgbModal,
    protected crudDomaineService: CruddomaineService,
    protected accountService: AccountService,
    protected eventManager: JhiEventManager,
    protected domaineService: DomaineService,
    protected bmcsBaseService: BmcsBaseService,
    protected storageService: StorageService,
    protected statistiqueService: StatistiqueService
  ) {
    super(accountService, statistiqueService, bmcsBaseService);
  }

  async ngOnInit(): Promise<void> {
    this.load = true;
    await super.ngOnInit();
    this.setDataBmcs();
    this.registerChangeInOperations();
    this.setBaseline();
    this.loadCrudDomaines();
  }

  setDataBmcs() {
    this.versionedSystems = this.getVersionedSystems;
    this.deployedCenters = this.getDeployedCenters;
  }

  setBaseline() {
    const bl = this.storageService.get(StoreKey.BCS_BASELINE) as Baseline;
    if (bl) this.baseline = bl;
    else this.baseline = this.isUserMoss ? Baseline.NONE : Baseline.VISIBLE_CLIENT;
    this.loadDataByBaseline();
  }

  loadDataByBaseline() {
    this.storageService.set(StoreKey.BCS_BASELINE, this.baseline);
    this.loadOperationByBaseline(this.baseline);
    this.loadMilestoneByBaseline(this.baseline);
  }

  loadOperationByBaseline(baseline: Baseline) {
    this.load = true;
    if (baseline == Baseline.NONE) this.loadAllOperations();
    else {
      this.operationService.queryByBaseline(baseline).subscribe((res: HttpResponse<IOperation[]>) => {
        this.operations = res.body || [];
        this.load = false;
      });
    }
  }

  loadAllOperations() {
    this.operationService.query(this.isAdminBCS).subscribe((res: HttpResponse<IOperation[]>) => {
      this.operations = res.body || [];
      this.load = false;
    });
  }

  loadAllMilestones() {
    this.milestoneService.query(this.isAdminBCS).subscribe((res: HttpResponse<IMilestone[]>) => {
      this.milestones = res.body || [];
    });
  }

  loadMilestoneByBaseline(baseline: Baseline) {
    if (baseline == Baseline.NONE) this.loadAllMilestones();
    else {
      this.milestoneService.queryByBaseline(baseline).subscribe((res: HttpResponse<IMilestone[]>) => {
        this.milestones = res.body || [];
      });
    }
  }

  loadCrudDomaines(): void {
    this.domains = this.getDomaines(true);
    if (this.isAdminBCS) {
      this.overseenDomains = [...this.domains];
    } else {
      this.crudDomaineService.queryMyne({ where: this.accountService.userGroup })
        .subscribe(
          (res: HttpResponse<[ICruddomaine[]]>) => {
            this.crudDomaines = res.body || [[]];
            const overseen = [];
            this.crudDomaines.forEach(crud => {
              crud.forEach(cd => {
                if (cd.is_referent) {
                  const domain = this.domains.find(d => d.id === cd.domaine?.id);
                  if (domain) overseen.push(domain);
                }
              });
            })
            this.overseenDomains = overseen;
          },
          (err) => console.error(err)
        );
    }
  }

  ngOnDestroy(): void {
    if (this.eventSubscriber)
      this.eventManager.destroy(this.eventSubscriber);
  }

  importBcsData(): void {
    this.modalService.open(ImportBcsDialogComponent, { size: 'lg', backdrop: 'static' });
  }

  registerChangeInOperations(): void {
    this.eventSubscriber = this.eventManager.subscribe('operationListModification', () => this.loadAllOperations());
  }

  setHistoricToBaseline(domain: IDomaine, newBaselineValue?: Baseline): void {
    const modalRef = this.modalService.open(BcsNgHistoricScopeUpdateComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.id = domain.id;
    modalRef.componentInstance.validationScope = RpsValidationScope.DOMAINE;
    modalRef.componentInstance.newBaselineValue = newBaselineValue;
  }
}